CREATE TABLE IF NOT EXISTS games
(
    game_id varchar (10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(game_id,'^[Gg][0123456789]+')),
    game_name varchar (100) NOT NULL,
    game_description varchar (500) NOT NULL,
    game_language varchar (10) NOT NULL,
    game_author varchar (100) NOT NULL,
    game_version numeric(4,2),
    game_text varchar (500) NOT NULL,
    game_first_room varchar (10),
    game_end_room varchar (10),
    game_initial_player_life_points integer default 0 NOT NULL,
    game_initial_player_attack integer default 0 NOT NULL,
    game_initial_player_money numeric (7,2) default 0.0 NOT NULL,
    game_initial_player_attack_speed integer default 0 NOT NULL
);

CREATE TABLE IF NOT EXISTS rooms
(
    room_id varchar (10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(room_id,'^[Rr][0123456789]+')),
    room_name varchar (100) NOT NULL,
    room_article varchar (10) NOT NULL,
    room_text varchar (500) NOT NULL,
    room_description varchar (500) NOT NULL,
    room_is_dark varchar (5) default 'false' NOT NULL
    check (room_is_dark = 'true' OR room_is_dark = 'false'),
    room_in_game varchar (10) NOT NULL FOREIGN KEY REFERENCES games (game_id)
);

CREATE TABLE IF NOT EXISTS room_map_coords
(
    room_id varchar (10) NOT NULL PRIMARY KEY FOREIGN KEY REFERENCES rooms(room_id),
    room_x_coord integer NOT NULL,
    room_y_coord integer NOT NULL,
    room_width integer NOT NULL,
    room_hight integer NOT NULL,
    player_point_x integer NOT NULL,
    player_point_y integer NOT NULL,
    player_point_width integer NOT NULL,
    player_point_hight integer NOT NULL,
    player_point_arc_width integer NOT NULL,
    player_point_arc_hight integer NOT NULL
);

CREATE TABLE IF NOT EXISTS paths
(
    path_from_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    path_to_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    path_direction varchar (2) NOT NULL
    check (path_direction = 'n' OR path_direction = 's' OR path_direction = 'e' OR path_direction = 'w'
    OR path_direction = 'ne' OR path_direction = 'nw' OR path_direction = 'se'
    OR path_direction = 'sw' OR path_direction = 'u' OR path_direction = 'd'),
    path_type varchar (100) NOT NULL,
    path_article varchar (10) NOT NULL,
    path_description varchar (500) NOT NULL,
    path_needs_object varchar (10),
    CONSTRAINT PK_paths PRIMARY KEY (path_from_room, path_direction)
);

CREATE TABLE IF NOT EXISTS path_map_coords
(
    path_from_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    path_to_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    path_x_coord integer NOT NULL,
    path_y_coord integer NOT NULL,
    path_width integer NOT NULL,
    path_hight integer NOT NULL,
    path_is_one_way varchar(5) NOT NULL
    check (path_is_one_way = 'true' OR path_is_one_way = 'false'),
    CONSTRAINT PK_path_map_coords PRIMARY KEY (path_from_room, path_to_room)
);

CREATE TABLE IF NOT EXISTS npcs
(
    npc_id varchar (10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(npc_id,'^[Nn][0123456789]+')),
    npc_name varchar (100) NOT NULL,
    npc_type varchar (100) NOT NULL,
    npc_article varchar (10) NOT NULL,
    npc_text varchar (500) NOT NULL,
    npc_description varchar (500) NOT NULL,
    npc_in_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    npc_is_merchant varchar (5) default 'false' NOT NULL
    check (npc_is_merchant = 'true' OR npc_is_merchant = 'false')
);

CREATE TABLE IF NOT EXISTS quests
(
    quest_id varchar (10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(quest_id,'^[Qq][0123456789]+')),
    quest_title varchar (100) NOT NULL,
    quest_text varchar (500) NOT NULL,
    quest_solution varchar (500) NOT NULL,
    quest_effect_when_solved varchar (500),
    quest_from_npc varchar (10) NOT NULL FOREIGN KEY REFERENCES npcs (npc_id)
);

CREATE TABLE IF NOT EXISTS quest_reward
(
    quest_id varchar (10) NOT NULL FOREIGN KEY REFERENCES quests (quest_id),
    quest_reward_id integer NOT NULL,
    quest_reward_type varchar(10) NOT NULL
        check (quest_reward_type = 'lp' OR quest_reward_type = 'attack' OR
               quest_reward_type = 'speed' OR quest_reward_type = 'money' OR
               quest_reward_type = 'object'),
    quest_reward_value integer,
    quest_reward_object varchar(10),
    quest_solution_type varchar(50) NOT NULL,
    quest_solution_subject varchar(50) NOT NULL,
    PRIMARY KEY (quest_id,quest_reward_id)
);

CREATE TABLE IF NOT EXISTS enemies
(
    game_id varchar(10) NOT NULL FOREIGN KEY REFERENCES games (game_id),
    check (REGEXP_MATCHES(game_id,'^[Gg][0123456789]+')),
    enemy_id varchar (10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(enemy_id,'^[Ee][0123456789]+')),
    enemy_name varchar (100) NOT NULL,
    enemy_type varchar (100) NOT NULL,
    enemy_article varchar (10) NOT NULL,
    enemy_text varchar (500) NOT NULL,
    enemy_description varchar (500) NOT NULL,
    enemy_life_points integer default 0 NOT NULL,
    enemy_attack integer default 0 NOT NULL,
    enemy_attack_speed integer default 0 NOT NULL,
    enemy_effect_when_defeated varchar (500),
    enemy_when_defeated_attackup integer default 0 NOT NULL,
    enemy_when_defeated_lifepointsup integer default 0 NOT NULL,
    enemy_when_defeated_money decimal(7,2) default 0 NOT NULL,
    enemy_when_defeated_speed integer default 0 NOT NULL,
    enemy_in_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id)
);

CREATE TABLE IF NOT EXISTS objects
(
    game_id varchar(10) NOT NULL FOREIGN KEY REFERENCES games (game_id),
    check (REGEXP_MATCHES(game_id,'^[Gg][0123456789]+')),
    object_id varchar (10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(object_id,'^[Oo][0123456789]+')),
    object_name varchar (100) NOT NULL,
    object_article varchar (10) NOT NULL,
    object_description varchar (500) NOT NULL,
    object_initial_value decimal(7,2) default 0 NOT NULL,
    object_eatable varchar (5) default 'false' NOT NULL
    check (object_eatable = 'true' OR object_eatable = 'false'),
    object_effect_when_eaten varchar (500),
    object_switchable varchar (5) default 'false' NOT NULL
    check (object_switchable = 'true' OR object_switchable = 'false'),
    object_magical varchar (5) default 'false' NOT NULL
    check (object_magical = 'true' OR object_magical = 'false'),
    object_effect_when_magical varchar (500),
    object_useable_cheats integer default 0 NOT NULL,
    object_attack integer default 0 NOT NULL,
    object_initial_place varchar (10)
    check (object_initial_place like 'r%' OR object_initial_place like 'R%' OR
    object_initial_place like 'c%' OR object_initial_place like 'C%' OR
    object_initial_place like 'n%' OR object_initial_place like 'N%' OR
    object_initial_place like 'e%' OR object_initial_place like 'E%'),
    object_initial_visible varchar(5) default 'false' NOT NULL
    check (object_initial_visible = 'true' OR object_initial_visible = 'false')
);

CREATE TABLE IF NOT EXISTS object_effects
(
    object_id varchar (10) NOT NULL FOREIGN KEY REFERENCES objects (object_id),
    object_effect_id integer NOT NULL,
    object_effect_from varchar(10) NOT NULL
    check (object_effect_from = 'eat' OR object_effect_from = 'use'),
    object_effect_type varchar(10) NOT NULL
    check (object_effect_type = 'lp' OR object_effect_type = 'attack' OR
           object_effect_type = 'speed' OR object_effect_type = 'money' OR
           object_effect_type = 'light'),
    object_effect_value integer,
    PRIMARY KEY (object_id,object_effect_id)
);

CREATE TABLE IF NOT EXISTS containers
(
    container_id varchar (10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(container_id,'^[Cc][0123456789]+')),
    container_name varchar (100) NOT NULL,
    container_article varchar (10) NOT NULL,
    container_description varchar (500) default 'Looks normal.' NOT NULL,
    container_needs_object varchar (10) FOREIGN KEY REFERENCES objects (object_id),
    container_in_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id)
);

CREATE TABLE IF NOT EXISTS game_users
(
    user_id varchar (10) NOT NULL PRIMARY KEY,
    user_password varchar (20),
    user_last_login TIMESTAMP
);

CREATE TABLE IF NOT EXISTS players
(
    player_id integer GENERATED BY DEFAULT AS IDENTITY
    (START WITH 1 INCREMENT BY 1) NOT NULL PRIMARY KEY,
    game_user_id varchar(10) NOT NULL FOREIGN KEY REFERENCES game_users (user_id),
    player_name varchar (20) NOT NULL,
    player_plays_game varchar (10) NOT NULL FOREIGN KEY REFERENCES games (game_id),
    player_last_login TIMESTAMP NOT NULL,
    player_life_points integer default 1 NOT NULL,
    player_attack integer default 0 NOT NULL,
    player_attack_speed integer default 0 NOT NULL,
    player_money decimal (7,2) default 0 NOT NULL,
    player_actions_left integer default 10 NOT NULL,
    player_show_statements varchar (5) default 'true' NOT NULL
    check (player_show_statements = 'true' OR player_show_statements = 'false'),
    player_in_room varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    player_table_style integer default 1 NOT NULL,
    player_theme_style integer default 1 NOT NULL,   -- dark mode 2 und white mode 1
    player_font_size integer default 14 NOT NULL
    check (player_font_size > 0)
);

CREATE TABLE IF NOT EXISTS container_examine
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    container_id varchar (10) NOT NULL FOREIGN KEY REFERENCES containers (container_id),
    container_is_examined varchar(5) default 'false' NOT NULL
    check (container_is_examined = 'true' OR container_is_examined = 'false'),
    container_is_visible varchar(5) default 'false' NOT NULL
    check (container_is_visible = 'true' OR container_is_visible = 'false'),
    CONSTRAINT unique_container_examine unique (player_id, container_id)
);

CREATE TABLE IF NOT EXISTS rooms_visited
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    room_id varchar (10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    CONSTRAINT PK_rooms_visited PRIMARY KEY (player_id, room_id)
);

CREATE TABLE IF NOT EXISTS objects_properties
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    object_id varchar (10) NOT NULL FOREIGN KEY REFERENCES objects (object_id),
    object_value decimal(7,2) default 0 NOT NULL,
    object_eaten varchar (5) default 'false' NOT NULL
    check (object_eaten = 'true' OR object_eaten = 'false'),
    object_switched_on varchar (5) default 'false' NOT NULL
    check (object_switched_on = 'true' OR object_switched_on = 'false'),
    object_magical_effect_used varchar (5) default 'false' NOT NULL
    check (object_magical_effect_used = 'true' OR object_magical_effect_used = 'false'),
    object_cheats_left integer default 0 NOT NULL,
    object_in_container varchar (10) FOREIGN KEY REFERENCES containers (container_id),
    object_in_room varchar (10) FOREIGN KEY REFERENCES rooms (room_id),
    object_from_npc varchar (10) FOREIGN KEY REFERENCES npcs (npc_id),
    object_from_enemy varchar (10) FOREIGN KEY REFERENCES enemies (enemy_id),
    object_in_inventory varchar (5) default 'false' NOT NULL
    check (object_in_inventory = 'true' OR object_in_inventory = 'false'),
    object_is_visible varchar (5) default 'false'
    check (object_is_visible = 'true' OR object_is_visible = 'false'),
    CONSTRAINT check_object_place check (
(object_in_container is not null and object_in_room is null and object_from_npc is null
    and object_from_enemy is null and object_in_inventory = 'false') OR
(object_in_container is null and object_in_room is not null and object_from_npc is null
    and object_from_enemy is null and object_in_inventory = 'false') OR
(object_in_container is null and object_in_room is null and object_from_npc is not null
    and object_from_enemy is null and object_in_inventory = 'false') OR
(object_in_container is null and object_in_room is null and object_from_npc is null
    and object_from_enemy is not null and object_in_inventory = 'false') OR
(object_in_container is null and object_in_room is null and object_from_npc is null
    and object_from_enemy is null and object_in_inventory = 'true')),
    CONSTRAINT PK_objects_properties PRIMARY KEY (player_id, object_id)
);

CREATE TABLE IF NOT EXISTS quests_received
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    quest_id varchar (10) NOT NULL FOREIGN KEY REFERENCES quests (quest_id),
    quest_received varchar(5) default 'false' NOT NULL
    check (quest_received = 'true' OR quest_received = 'false'),
    quest_solved varchar (5) default 'false' NOT NULL
    check (quest_solved = 'true' OR quest_solved = 'false'),
    quest_reward_received varchar (5) default 'false' NOT NULL
    check (quest_reward_received = 'true' OR quest_reward_received = 'false'),
    quest_visible varchar(5) default 'false' NOT NULL
    check (quest_visible = 'true' OR quest_visible = 'false'),
    CONSTRAINT PK_quests_received PRIMARY KEY (player_id, quest_id)
);

CREATE TABLE IF NOT EXISTS enemy_fights
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    enemy_id varchar (10) NOT NULL FOREIGN KEY REFERENCES enemies (enemy_id),
    enemy_current_life_points integer NOT NULL,
    enemy_stored_player_money integer default 0 NOT NULL,
    enemy_defeated varchar (5) default 'false' NOT NULL
    check (enemy_defeated = 'true' OR enemy_defeated = 'false'),
    CONSTRAINT PK_enemy_fights PRIMARY KEY (player_id, enemy_id)
);

CREATE TABLE IF NOT EXISTS enemy_attack
(
    player_id integer,
    enemy_id varchar (10) NOT NULL FOREIGN KEY REFERENCES enemies (enemy_id),
    object_id varchar (10) FOREIGN KEY REFERENCES objects (object_id)
);

CREATE TABLE IF NOT EXISTS actions
(
    action_id integer NOT NULL PRIMARY KEY,
    action varchar (100) NOT NULL,
    action_type varchar(10) NOT NULL
    check (action_type = 'gui' OR action_type = 'adventure'),
    action_sql_command varchar(2000) NOT NULL,
    action_replace_words varchar(1000) NOT NULL,
    action_attribute_count integer NOT NULL,
    action_description varchar (500) NOT NULL,
    action_language varchar (10) NOT NULL,
    action_attribute_names varchar(200),
    action_with_table_output varchar(5) default 'false' NOT NULL
    check (action_with_table_output = 'true' OR action_with_table_output = 'false')
);

CREATE TABLE IF NOT EXISTS actions_sequences
(
    action_id integer NOT NULL,
    action_sequence_id integer NOT NULL,			-- Reihenfolge der Ausführung
    action_sql_command varchar(2000) NOT NULL,		-- sql Request
    action_connection varchar(5),				    -- gcon or pcon
    action_type varchar(50) NOT NULL,			    -- z.B. select, select out, select execute, update, insert, output, (change), check, jump, select check execute, select check out
    action_need_attribute varchar(100) NOT NULL,    -- Komma getrennt Reihenfolge von Attributen (z.B. 1,2,1,s3.1 für 1.Att. ,2. Att., 1. Att., Seq. 3 - 1. Att.)
    action_output varchar(1000),				    -- String für Ausgabe (Attribute durch ? Kennzeichnen) (aber auch für bestimmte andere Typen in Benutzung)
    action_select_count integer default 0 NOT NULL,	-- Angabe wie viele Attribute select werden (bei kein select statment = 0)
    action_check_attribute_with varchar(200),		-- check needAttribute equal einen angegebenen String
    action_jump_to integer,					        -- Angabe wohin das Programm springen soll (zu welcher sequence_id)
    CONSTRAINT PK_ACTIONS PRIMARY KEY (action_id,action_sequence_id)
);

CREATE TABLE IF NOT EXISTS command_history
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    command_id integer NOT NULL,
    command_given varchar (2000) NOT NULL,
    command_as_action varchar (250) NOT NULL,
    CONSTRAINT PK_command_history PRIMARY KEY (player_id, command_id)
);

CREATE TABLE IF NOT EXISTS output_history
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    output_id integer NOT NULL,
    output varchar(5000) NOT NULL,
    output_time TIMESTAMP NOT NULL,
    output_is_last BOOLEAN NOT NULL,
    CONSTRAINT PK_output_history Primary Key (player_id,output_id)
);

CREATE TABLE IF NOT EXISTS trade_offers
(
    player_id integer,
    npc_id varchar (10) NOT NULL FOREIGN KEY REFERENCES npcs(npc_id),
    object_id varchar (10) NOT NULL FOREIGN KEY REFERENCES objects (object_id),
    offer_value decimal(7,2) NOT NULL,
    CONSTRAINT Unique_trade_offers_player_id_object_id unique (player_id,object_id)
);

CREATE TABLE IF NOT EXISTS trashs
(
    trash_id varchar(10) NOT NULL PRIMARY KEY
    check (REGEXP_MATCHES(trash_id,'^[Tt][0123456789]+')),
    trash varchar(50) NOT NULL,
    trash_in_room varchar(10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id)
);

CREATE TABLE IF NOT EXISTS trash_report
(
    player_id integer NOT NULL FOREIGN KEY REFERENCES players (player_id),
    trash_id varchar(10) NOT NULL FOREIGN KEY REFERENCES trashs (trash_id),
    trash varchar(50) NOT NULL,
    trash_in_room varchar(10) NOT NULL FOREIGN KEY REFERENCES rooms (room_id),
    trash_is_visible varchar(5) default 'false' NOT NULL
    check (trash_is_visible = 'true' OR trash_is_visible = 'false'),
    CONSTRAINT PK_trash_report Primary Key (player_id,trash_id)
);

-- alter table
ALTER TABLE games
    ADD FOREIGN KEY (game_first_room) REFERENCES rooms (room_id);
ALTER TABLE games
    ADD FOREIGN KEY (game_end_room) REFERENCES rooms (room_id);
ALTER TABLE paths
    ADD FOREIGN KEY (path_needs_object) REFERENCES objects (object_id);

-- create views
CREATE VIEW IF NOT EXISTS trade_offer AS
SELECT npc_id, object_id, offer_value
FROM trade_offers
WHERE player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS game AS
SELECT g.game_name, g.game_description, g.game_author, g.game_version, g.game_text
FROM games g, players p
WHERE g.game_id = p.player_plays_game AND
        p.player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS room AS
SELECT r.room_id, r.room_name, r.room_text, r.room_description, r.room_is_dark
FROM rooms r, rooms_visited rv
WHERE r.room_id = rv.room_id AND
        rv.player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS path AS
SELECT DISTINCT p.path_from_room, r.room_name AS path_from_room_name, p.path_to_room,
                p.path_direction, p.path_type, p.path_description,
                CASE
                    WHEN p.path_needs_object is null THEN 'None'
                    WHEN p.path_needs_object is not null THEN o.object_name
                    END AS path_needs_object
FROM paths p, rooms_visited rv, objects o, rooms r
WHERE rv.player_id = CURRENT_USER AND rv.room_id = p.path_from_room AND
    (p.path_needs_object is null OR o.object_id = p.path_needs_object) AND
        r.room_id = p.path_from_room;

CREATE VIEW IF NOT EXISTS npc AS
SELECT n.npc_id, n.npc_name, n.npc_type, n.npc_text, n.npc_description, n.npc_in_room,
       n.npc_is_merchant
FROM npcs n, rooms_visited rv
WHERE n.npc_in_room = rv.room_id AND
        rv.player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS quest AS
SELECT q.quest_id, q.quest_title, q.quest_text, q.quest_from_npc, qr.quest_received, qr.quest_solved, qr.quest_reward_received
FROM quests q, quests_received qr
WHERE q.quest_id = qr.quest_id AND
        qr.player_id = CURRENT_USER
        and qr.quest_visible = 'true';

CREATE VIEW IF NOT EXISTS quest_update AS
SELECT quest_id, quest_received, quest_solved, quest_reward_received
FROM quests_received
WHERE player_id = CURRENT_USER and
      quest_visible = 'true';

CREATE VIEW IF NOT EXISTS enemy AS
SELECT e.enemy_id, e.enemy_name, e.enemy_text, e.enemy_description, e.enemy_life_points,
       e.enemy_attack, e.enemy_attack_speed, e.enemy_in_room, ef.enemy_current_life_points, ef.enemy_stored_player_money, ef.enemy_defeated,
       e.enemy_effect_when_defeated
FROM enemies e, enemy_fights ef, rooms_visited rv
WHERE e.enemy_id = ef.enemy_id AND
      e.enemy_in_room = rv.room_id AND
      rv.player_id = ef.player_id AND
      ef.player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS enemy_history AS
SELECT enemy_id, object_id
FROM enemy_attack
WHERE player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS object AS
SELECT o.object_id, o.object_name, o.object_description, o.object_eatable,
       o.object_effect_when_eaten, o.object_switchable, object_magical, object_effect_when_magical,
       o.object_attack
FROM objects o, objects_properties op, players p
WHERE o.object_id = op.object_id AND
        op.object_is_visible = 'true' AND
        op.player_id = CURRENT_USER AND
        op.player_id = p.player_id AND
        p.player_plays_game = o.game_id;

CREATE VIEW IF NOT EXISTS room_inventory AS
SELECT DISTINCT c.container_id AS inventory_id, c.container_name AS inventory_name,
                c.container_article AS inventory_article, c.container_description AS inventory_description,
                c.container_in_room AS inventory_in_room,
                CASE
                    WHEN c.container_needs_object is null THEN 'None'
                    WHEN c.container_needs_object is not null THEN o.object_name
                    END AS inventory_needs_object
FROM containers c, rooms_visited rv, objects o
WHERE c.container_in_room = rv.room_id AND
    (c.container_needs_object is null OR c.container_needs_object = o.object_id) AND
        rv.player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS room_inv_examine AS
SELECT container_id as inventory_id, container_is_examined as inventory_is_examined
FROM container_examine
WHERE player_id = CURRENT_USER and container_is_visible = true;

CREATE VIEW IF NOT EXISTS player AS
SELECT player_id, player_name, player_life_points, player_attack, player_attack_speed, player_money, player_actions_left,
       player_in_room, player_show_statements, player_table_style, player_theme_style, player_font_size, player_plays_game,
       player_last_login
FROM players
WHERE player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS object_properties AS
SELECT op.object_id, op.object_value, op.object_eaten, op.object_switched_on, op.object_magical_effect_used, op.object_in_container, op.object_in_room,
       op.object_from_npc, op.object_in_inventory, op.object_cheats_left
FROM objects_properties op
WHERE op.player_id = CURRENT_USER AND
        op.object_is_visible = 'true';

CREATE VIEW IF NOT EXISTS actions_english AS
SELECT action_id, action, action_sql_command, action_description, action_type
FROM actions
WHERE action_language = 'english';

CREATE VIEW IF NOT EXISTS actions_deutsch AS
SELECT action_id, action, action_sql_command, action_description, action_type
FROM actions
WHERE action_language = 'deutsch';

CREATE VIEW IF NOT EXISTS history AS
SELECT command_id, command_given, command_as_action
FROM command_history
WHERE player_id = CURRENT_USER;

CREATE VIEW IF NOT EXISTS output AS
SELECT output_id, output, output_time, output_is_last
FROM output_history
WHERE player_id = CURRENT_USER
ORDER BY output_id;

CREATE VIEW IF NOT EXISTS trash AS
SELECT trash_id, trash, trash_in_room from trash_report
where CURRENT_USER = player_id and
      trash_is_visible = 'true';

-- access rights
CREATE ROLE player;
GRANT SELECT ON game TO player;
GRANT SELECT ON room TO player;
GRANT SELECT ON path TO player;
GRANT SELECT ON npc TO player;
GRANT SELECT ON quest TO player;
GRANT SELECT, UPDATE (quest_received, quest_reward_received) ON quest_update TO player;
GRANT SELECT ON enemy TO player;
GRANT SELECT ON object TO player;
GRANT SELECT ON room_inventory TO player;
GRANT SELECT, UPDATE (inventory_is_examined) ON room_inv_examine TO player;
GRANT SELECT, UPDATE(player_name, player_in_room, player_show_statements
                    , player_table_style, player_theme_style, player_font_size) ON player TO player;
GRANT SELECT, UPDATE(object_eaten, object_switched_on, object_magical_effect_used, object_in_container
    , object_in_room, object_from_npc, object_in_inventory) ON object_properties TO player;
GRANT SELECT ON actions_english TO player;
GRANT SELECT ON actions_deutsch TO player;
GRANT SELECT ON history TO player;
GRANT SELECT ON output TO player;
GRANT SELECT, INSERT (npc_id, object_id, offer_value) ON trade_offer TO player;
GRANT SELECT, INSERT ON enemy_history TO player;
GRANT SELECT, DELETE ON trash TO player;
CREATE ROLE cheater;
GRANT SELECT ON games TO cheater;
GRANT SELECT ON rooms TO cheater;
GRANT SELECT ON paths TO cheater;
GRANT SELECT ON npcs TO cheater;
GRANT SELECT ON quests TO cheater;
GRANT SELECT ON enemies TO cheater;
GRANT SELECT ON objects TO cheater;
GRANT SELECT ON containers TO cheater;
GRANT SELECT ON quest_reward TO cheater;
GRANT SELECT ON object_effects TO cheater;
GRANT SELECT ON trashs TO cheater;

-- Insert
-- game
INSERT INTO games (game_id, game_name, game_description, game_language, game_author,
                   game_version, game_text, game_first_room,game_end_room, game_initial_player_life_points,
                   game_initial_player_attack, game_initial_player_money, game_initial_player_attack_speed)
VALUES ('g1', 'SampleGame', 'This game is used to test this application.', 'english', 'Isabell Roesner (original game),Tom Schindler (game extension)','2.0',
'You have a semester break. Going on a vacation would be great. But you are a little broke, so you decide to visit a campsite near the mountains.',null, null, 100, 3, 10, 16);

--rooms
INSERT INTO rooms (room_id, room_name, room_article, room_text, room_description,
                   room_is_dark, room_in_game)
VALUES ('r1', 'campsite', 'the', 'You see a lot abandoned tents. The other campers already went on a hike probably. You feel a thirst for action.',
        'A small camp near the mountains. The pictures on the website were not lying.', default, 'g1');
INSERT INTO rooms
VALUES ('r2', 'beautiful meadow', 'a', 'It is quiet, green and spacious.',
        'There are birds chirping around you. Butterflies flutter past you. Some people have a picnic afar.', default, 'g1');
INSERT INTO rooms
VALUES ('r3', 'forest', 'a', 'The trail ends here. There are so many trees. You have a little fear of getting lost. Luckily, your phone has GPS.',
        'You see trees, bushes and mushrooms. It is just a forest.', default, 'g1');
INSERT INTO rooms
VALUES ('r4', 'dense forest', 'a', 'Actually, you only see lots of trees now.',
        'How could the forest become even denser?', default, 'g1');
INSERT INTO rooms
VALUES ('r5', 'dark cave', 'a', 'It is very cramped and dark. You have a bad feeling about this and want to leave as quickly as possible actually.',
        'From the high ceiling some drops are trickling. You hope that there are no dangerous animals in here.', 'true', 'g1');
INSERT INTO rooms
VALUES ('r6', 'end room', 'the', 'As you arrive at the campsite it became dark already. You search your tent and go to sleep. That was an interesting day. So you finished this test game.',
        'The end.',default, 'g1');
INSERT INTO rooms
VALUES ('r7', 'inner dark cave', 'the', 'It is very dark. You can hear a cry for help deeper in the cave.',
        '','true', 'g1');
INSERT INTO rooms
VALUES ('r8', 'end of the cave', 'the', 'From a corner, a light source seems to light up the room a little bit. You can faintly see a person lying in a corner calling for help. You have a bad feeling.',
        'You are in a dead end from the cave. You see a frightened person that needs your help. In the dark corner appears a large silhouette.',default, 'g1');
INSERT INTO rooms
VALUES ('r9', 'farm', 'a', 'The trail ends here. There is a little farm with animals and grown plants.',
        'Wheat is grown in the fields. Everywhere chickens run freely on the farm.',default, 'g1');
INSERT INTO rooms
VALUES ('r10', 'sea', 'the', 'At the end of the path you come to a beautiful small beach by a sea.',
        'It is a small beach by the sea surrounded by cliffs where some people swim. Maybe I can fish near the cliffs.',default, 'g1');

--objects
INSERT INTO objects (game_id,object_id, object_name, object_article, object_description, object_initial_value,
                     object_eatable, object_effect_when_eaten,object_switchable,
                     object_magical, object_effect_when_magical,object_useable_cheats,
                     object_attack, object_initial_place, object_initial_visible) VALUES
    ('g1','o1', 'water bottle', 'a', 'It will refresh you.', 1, 'true', 'lpup 20',default, default, null, default,0, null,default),
    ('g1','o2', 'bag of chips', 'a', 'Delicious potato chips with paprika flavor.', 1, 'true', 'lpup 5',default, default, null,  default, 0, null,default),
    ('g1','o3', 'map of the region', 'a', 'There are many trails marked.', 6, default, null, default,
        default, null, default, 0, null,default),
    ('g1','o4', 'key to the end of the game', 'the', 'Opens the way to the end of the game.', 0, default,
        null, default, 'true', 'attup 50 lpup 100', default, 0, null,default),
    ('g1','o5', 'stone', 'a', 'A normal stone.', 1, default, null, default, default, null, default, 5, null,default),
    ('g1','o6', 'stick', 'a', 'A normal stick. It may leaven your hike.', 1, default, null, default, default,
        null, default, 35,null,default),
    ('g1','o7', 'big leaf', 'a', 'An especially big leaf.', 0, default, null, default, default, null, 20, 5, null,default),
    ('g1','o8', 'flashlight', 'a', 'It already contains batteries. Do not forget to switch it on.', 30,
        default, null, 'true', 'true', null, default, 10, null,default),
    ('g1','o9', 'bag of chips', 'a', 'Delicious potato chips with salt and vinegar flavor.', 1, 'true', 'lpup 5', default, default, null, default, 0, null,default),
    ('g1','o10', 'blanket', 'a', 'You can lay it on the grass and enjoy the peaceful atmosphere.', 20,
        default, null, default, 'true', 'lpup 60', default, 0, null,default),
    ('g1','o11', 'peach','a','Mario is not in sight.',2,'true','lpup 10 speed 5',default,default,null,default,1,null,default),
    ('g1','o12', 'ladder','a','A old wood ladder.',5,default,null,default,default,null,default,0,null,default),
    ('g1','o13', 'treasure key','a','A key to a supposed treasure.',5,default,null,default,default,null,default,0,null,default),
    ('g1','o14', 'sword','a','A magical stone sword. Maybe it''s good for fights.',30,default,null,default,'true','lpup 40 speed 5',default,70,null,default),
    ('g1','o15', 'bread','a','A freshly baked bread.',8,'true','lpup 30 attup 5',default,default,null,default,0,null,default),
    ('g1','o16', 'egg','a','A egg from a chicken.',5,'true','lpup 20 attup 3',default,default,null,default,0,null,default);

-- object_effects
INSERT INTO OBJECT_EFFECTS (object_id, object_effect_id, object_effect_from, object_effect_type, object_effect_value) VALUES
    ('o1',1,'eat','lp',20), ('o2',1,'eat','lp',5), ('o4',1,'use','lp',100),
    ('o4',2,'use','attack',50), ('o8',1,'use','light',null),('o9',1,'eat','lp',5), ('o10',1,'use','lp',60),
    ('o11',1,'eat','lp',10),('o11',2,'eat','speed',5),('o14',1,'use','lp',40),('o14',2,'use','speed',5),
    ('o15',1,'eat','attack',5),('o15',2,'eat','lp',30),('o16',1,'eat','attack',3),('o16',2,'eat','lp',20);

--paths
INSERT INTO paths (path_from_room, path_to_room, path_direction, path_type, path_article,
                   path_description, path_needs_object)
VALUES ('r1', 'r6', 's', 'way out of this game', 'a', 'Take this path, if you like to end this game.', 'o4');
INSERT INTO paths VALUES ('r6', 'r1', 'n', 'way in this game', 'a', 'Take this path, if you like to go to this game.', 'o4');
INSERT INTO paths VALUES ('r1', 'r2', 'n', 'path', 'a', 'A paved path.', null);
INSERT INTO paths VALUES ('r2', 'r1', 's', 'path', 'a', 'A paved path.', null);
INSERT INTO paths
VALUES ('r1', 'r3', 'e', 'trail', 'a', 'The path leads to the forest. You cannot see its end.', null);
INSERT INTO paths VALUES ('r3', 'r1', 'w', 'trail', 'a', 'The path leads out of the forest.', null);
INSERT INTO paths
VALUES ('r3', 'r4', 'e', 'path between the trees', 'a', 'You can see through. But actually there are only more trees.', null);
INSERT INTO paths
VALUES ('r4', 'r3', 'w', 'path between the trees', 'a', 'You can see through. But actually there are only more trees. Hopefully you do not run in circles.', null);
INSERT INTO paths VALUES ('r4', 'r5', 'n', 'mountain path', 'a', 'A path along the mountain.', null);
INSERT INTO paths VALUES ('r5', 'r4', 's', 'mountain path', 'a', 'A path along the mountain.', null);
INSERT INTO paths
VALUES ('r5', 'r2', 'w', 'small passage', 'a', 'A small passage you may easily overlook. But you could squeeze through.', null);
INSERT INTO paths
VALUES ('r5', 'r7', 'd', 'small cave crevice', 'a', 'A small cave crevice you might fit through, but you need something to climb down.', 'o12');
INSERT INTO paths
VALUES ('r7', 'r5', 'u', 'small cave crevice', 'a', 'A small cave crevice you might fit through, but you need something to climb up.', 'o12');
INSERT INTO paths
VALUES ('r7', 'r8', 'd', 'small cave path', 'a', 'A deeper-going cave path. Maybe you could squeeze through.', null);
INSERT INTO paths
VALUES ('r8', 'r7', 'u', 'small cave path', 'a', 'A small cave path back. Maybe you could squeeze through.', null);
INSERT INTO paths
VALUES ('r4', 'r10', 'e', 'path', 'a', 'A path between the forest and the sea.', null);
INSERT INTO paths
VALUES ('r10', 'r4', 'w', 'path', 'a', 'A path between the forest and the sea.', null);
INSERT INTO paths
VALUES ('r4', 'r9', 's', 'trail', 'a', 'A beautiful dirt trail lead out of the forest.', null);
INSERT INTO paths
VALUES ('r9', 'r4', 'n', 'trail', 'a', 'A beautiful dirt trail lead to a forest.', null);

--npcs
INSERT INTO npcs (npc_id, npc_name, npc_type, npc_article, npc_text, npc_description,
                  npc_in_room, npc_is_merchant)
VALUES ('n1', 'John', 'camper', 'a', 'My wife only brought along healthy stuff. What would I give for a bag of chips now...',
'A corpulently man. He is sitting on a blanket and looks unhappy.', 'r2',default);
INSERT INTO npcs
VALUES ('n2', 'Eve', 'camper', 'a', 'What a beautiful weather', 'A smiling woman. She sits on a blanket next to a man and eats some grapes.', 'r2', default);
INSERT INTO npcs
VALUES ('n3', 'Ann', 'old lady', 'an', 'The forest is so beautiful. I would like to go on a hike in the mountains, too, but I am afraid to get lost. If I had only brought a map...',
'An old lady. She looks very kind.', 'r3', default);
INSERT INTO npcs
VALUES ('n4', 'Mark', 'hiker', 'a', 'I heard there is a cave nearby, but I could not manage to find it.',
'A tall man. He has a backpack in his hand and looks as if he would leave for a hike soon, too.', 'r1',default);
INSERT INTO npcs
VALUES ('n5', 'Ben', 'cave explorer', 'a', 'I love exploring caves! But oh my god, that fat bat is looking at me the whole time. I am a little afraid it is going to attack me.',
'A man with a headlamp. In in such a cave it is definitely useful.', 'r5', default);
INSERT INTO npcs
VALUES ('n6', 'Mr Cook', 'owner of the camp', 'the', 'I hope you like my campsite. Come to me, if you need something.',
'A kind man. He also runs a small shop on his campsite.', 'r1', 'true');
INSERT INTO npcs
VALUES ('n7', 'Emil', 'cave explorer', 'a', 'Can you ple...ase help me, there is some...thing in the dark co..rner of the room tha..t scares me!',
'A scared man. He lies on the cave ground and calls for help.', 'r8',default);
INSERT INTO npcs
VALUES ('n8', 'Olivia', 'farmer', 'a', 'Howdy, you are welcome on my little farm. Take a look around. Come to me, if you need something.',
        'A hard working farmer. She also runs a small farmer store.', 'r9','true');
INSERT INTO npcs
VALUES ('n9', 'Noah', 'beach visitor', 'a', 'Hello, I actually wanted to swim but I''m disgusted by the garbage lying on the beach.',
        'A beach visitor who is disgusted by the garbage lying around.', 'r10',default);

--quests
INSERT INTO quests (quest_id, quest_title, quest_text, quest_solution, quest_effect_when_solved,
                    quest_from_npc)
VALUES ('q1', 'The hungry camper', 'In exchange for a bag of chips he may give you his map.',
        'npcobjname n1 bag of chips', 'map of the region', 'n1');
INSERT INTO quests
VALUES ('q2', 'Do not get lost', 'The old lady would be happy about a map of the mountain trails for
sure.', 'npcobjid n3 o3', 'flashlight', 'n3');
INSERT INTO quests
VALUES ('q3', 'The hidden cave', 'The hiker would like to know how to get to the cave. If you find a
way, you could tell him about it.', 'room r5', '5 money', 'n4');
INSERT INTO quests
VALUES ('q4', 'The fat bat', 'The cave exporer is afraid of the fat bat, that stares at everyone. Maybe
you could show him that the bat is not that powerful.', 'enemy e2', 'ladder', 'n5');
INSERT INTO quests
VALUES ('q5', 'The large silhouette', 'The cave exporer is afraid of a large silhouette, maybe a dangerous animal. Help Emiland and see what''s hiding behind the silhouette. Be careful.',
'enemy e3', 'key to the end of the game', 'n7');
INSERT INTO quests
VALUES ('q6', 'Clean the world', 'The beach visitor would be happy, if you will remove all trash laying around. There are 5 trash things in the world. Find them and delete them.',
        'delete all trash', '5 money treasure key', 'n9');

-- quest_reward
INSERT INTO quest_reward (quest_id, quest_reward_id, quest_reward_type, quest_reward_value, quest_reward_object, quest_solution_type, quest_solution_subject) VALUES
('q1',1,'object',null,'o3','npc_need_object','n1,o2|o9'), ('q2',1,'object',null,'o8','npc_need_object','n3,o3'),
('q3',1,'money',5,null,'room_visited','r5'), ('q4',1,'object',null,'o12','enemy','e2'), ('q5',1,'object',null,'o4','enemy','e3'),
('q6',1,'object',null,'o13','trash','all'), ('q6',2,'money',5,null,'trash','all');

--containers
INSERT INTO containers (container_id, container_name, container_article, container_description,
                        container_needs_object, container_in_room)
VALUES ('c1', 'tent', 'your', 'It took pretty long building it. But who needs instructions.', null, 'r1');
INSERT INTO containers VALUES ('c2', 'apple tree', 'an', default, null, 'r4');
INSERT INTO containers VALUES ('c3', 'tree', 'a', 'A big tree.', null, 'r4');
INSERT INTO containers
VALUES ('c4', 'interesting high branch', 'an', 'It is high up. Maybe you can reach it with a stick.', 'o6','r3');
INSERT INTO containers
VALUES ('c5', 'treasure chest', 'a', 'A locked chest. You need a key.', 'o13','r7');

--enemies
INSERT INTO enemies (game_id,enemy_id, enemy_name, enemy_type, enemy_article, enemy_text,
                     enemy_description, enemy_life_points, enemy_attack, enemy_attack_speed,
                     enemy_effect_when_defeated, enemy_when_defeated_attackup, enemy_when_defeated_lifepointsup,
                     enemy_when_defeated_money, enemy_when_defeated_speed, enemy_in_room)
VALUES ('g1','e1', 'Teethy', 'bat', 'a', 'brz brz', 'A bat, that is hanging at the ceiling. How cool would it be
if it transformed into a vampire now.',60, 10, 15, 'lpup 50 attup 10 money 5 speed 5', 10, 50, 5, 5,'r5');
INSERT INTO enemies
VALUES ('g1','e2', 'Bert', 'bat', 'a', 'brzzz', 'A fat bat, that is staring at you. Oh no, how disgusting!',
        120, 20, 20, 'money 10 attup 10 speed 5', 10, 50, 10, 5,'r5');
INSERT INTO enemies
VALUES ('g1','e3', 'Balto', 'bear', 'a', 'brumm', 'A big bear, that is hidding in the dark. Very dangerous be well prepared if you get close to him.',
        250, 30, 25, 'money 20 attup 10 lpup 50 speed 5', 10, 50, 20, 5,'r8');

--trashs
INSERT INTO TRASHS (trash_id, trash, trash_in_room) VALUES
('t1','old can','r10'), ('t2','plastic bag','r1'),('t3','rotten meat','r5'),
('t4','old fishing net','r10'), ('t5','ripped newspaper','r2');

--object_initial_place updates
UPDATE games
SET game_first_room = 'r1', game_end_room = 'r6'
WHERE game_id = 'g1';
UPDATE games
SET game_first_room = 'r1', game_end_room = 'r2'
WHERE game_id = 'g2';
UPDATE objects
SET object_initial_place = 'c1'
WHERE object_id = 'o1';
UPDATE objects
SET object_initial_place = 'c1'
WHERE object_id = 'o2';
UPDATE objects
SET object_initial_place = 'n1'
WHERE object_id = 'o3';
UPDATE objects
SET object_initial_place = 'n7'
WHERE object_id = 'o4';
UPDATE objects
SET object_initial_place = 'r5'
WHERE object_id = 'o5';
UPDATE objects
SET object_initial_place = 'r4'
WHERE object_id = 'o6';
UPDATE objects
SET object_initial_place = 'c4'
WHERE object_id = 'o7';
UPDATE objects
SET object_initial_place = 'n3'
WHERE object_id = 'o8';
UPDATE objects
SET object_initial_place = 'n6'
WHERE object_id = 'o9';
UPDATE objects
SET object_initial_place = 'n6'
WHERE object_id = 'o10';
UPDATE objects
SET object_initial_place = 'c3'
WHERE object_id = 'o11';
UPDATE objects
SET object_initial_place = 'n5'
WHERE object_id = 'o12';
UPDATE objects
SET object_initial_place = 'n9'
WHERE object_id = 'o13';
UPDATE objects
SET object_initial_place = 'c5'
WHERE object_id = 'o14';
UPDATE objects
SET object_initial_place = 'n8'
WHERE object_id = 'o15';
UPDATE objects
SET object_initial_place = 'n8'
WHERE object_id = 'o16';

-- room_map_coords
Insert into room_map_coords (room_id, room_x_coord, room_y_coord, room_width, room_hight, player_point_x, player_point_y, player_point_width,
                             player_point_hight, player_point_arc_width, player_point_arc_hight) values
    ('r1',10,150,70,100,35,190,20,20,20,20),
    ('r2',10,10,70,100,35,50,20,20,20,20),
    ('r3',120,150,70,100,145,190,20,20,20,20),
    ('r4',230,150,70,100,255,190,20,20,20,20),
    ('r5',230,10,70,100,255,50,20,20,20,20),
    ('r6',10,290,70,100,35,330,20,20,20,20),
    ('r7',340,10,70,100,365,50,20,20,20,20),
    ('r8',450,10,70,100,475,50,20,20,20,20),
    ('r9',230,290,70,100,255,330,20,20,20,20),
    ('r10',340,150,70,100,365,190,20,20,20,20);

-- path_map_coords
INSERT into path_map_coords (path_from_room, path_to_room, path_x_coord, path_y_coord, path_width, path_hight, path_is_one_way) values
    ('r1','r2',40,110,10,40,'false'),        -- R1 - R2
    ('r1','r3',80,200,40,10,'false'),        -- R1 - R3
    ('r1','r6',40,250,10,40,'false'),        -- R1 - R6
    ('r3','r4',190,200,40,10,'false'),       -- R3 - R4
    ('r4','r5',260,110,10,40,'false'),       -- R4 - R5
    ('r5','r2',80,50,150,10,'true'),         -- R5 -> R2
    ('r4','r9',260,250,10,40,'false'),       -- R4 - R9
    ('r4','r10',300,200,40,10,'false'),      -- R4 - R10
    ('r5','r7',300,50,40,10,'false'),        -- R5 - R7
    ('r7','r8',410,50,40,10,'false');        -- R7 - R8

-- action commands english
INSERT INTO ACTIONS VALUES(1,'help','adventure','SELECT action, action_description FROM actions_english ORDER BY action_id asc;','',0,'Returns the actions of the game and a description how to use them.','english',null,'true');

INSERT INTO ACTIONS VALUES(2,'go','adventure','UPDATE player SET player_in_room = (SELECT pa.path_to_room FROM path pa, player p WHERE p.player_in_room = pa.path_from_room AND LOWER(pa.path_direction) = LOWER(?));'
                              ,'go,to',1,
                           '''go (to) <path_direction> / go <path_direction>'' Follow a path to another place. Changes your players.player_in_room value. Possible directions are north: ''n'', south: ''s'', west: ''w'', east: ''e'', and up: ''u'' and down: ''d''.','english','direction',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(2,1,'UPDATE player SET player_in_room = (SELECT pa.path_to_room FROM path pa, player p WHERE p.player_in_room = pa.path_from_room AND LOWER(?) like LOWER(pa.path_direction + ''%''));','pcon','update','1',null,0,null,null);

INSERT INTO ACTIONS VALUES(3,'examine','adventure','','examine',1,
                           '''examine <subject_name>''\''examine <subject_id>'' Get more information about an subject you can reach. Returns the description of the examined.','english','ID or name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(3,1,'SELECT TOP 1 o.object_description, ''true'', object_id
        FROM object o, object_properties op, player p
        WHERE o.object_id = op.object_id AND (LOWER(o.object_name) = LOWER(?) OR LOWER(op.object_id) = LOWER(?)) AND (op.object_in_inventory = ''true'' OR op.object_in_room = p.player_in_room);','pcon','select','1,1',null,3,null,null),
(3,2,'check',null,'check','s1.2',null,0,'true',15),
(3,3,'SELECT r.room_description, ''true'', r.room_id FROM room r, player p WHERE r.room_id = p.player_in_room and (Lower(r.room_id) = Lower(?) or Lower(r.room_name) = Lower(?));','pcon','select','1,1',null,3,null,null),
(3,4,'check',null,'check','s3.2',null,0,'true',17),
(3,5,'SELECT pa.path_description, ''true'', pa.path_direction FROM player p, path pa WHERE p.player_in_room = pa.path_from_room AND LOWER(pa.path_direction) = LOWER(?);','pcon','select','1',null,3,null,null),
(3,6,'check',null,'check','s5.2',null,0,'true',19),
(3,7,'SELECT n.npc_description, ''true'', n.npc_id FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND (LOWER(n.npc_name) = LOWER(?) or LOWER(n.npc_id) = LOWER(?));','pcon','select','1,1',null,3,null,null),
(3,8,'check',null,'check','s7.2',null,0,'true',21),
(3,9,'SELECT e.enemy_description, ''true'', e.enemy_id FROM enemy e, player p WHERE e.enemy_in_room = p.player_in_room AND (LOWER(e.enemy_name) = LOWER(?) or LOWER(e.enemy_id) = LOWER(?));','pcon','select','1,1',null,3,null,null),
(3,10,'check',null,'check','s9.2',null,0,'true',23),
(3,11,'SELECT ri.inventory_id ,ri.inventory_description, ''true'' FROM room_inventory ri, player p WHERE ri.inventory_in_room = p.player_in_room AND (LOWER(ri.inventory_name) = LOWER(?) or LOWER(ri.inventory_id) = LOWER(?));','pcon','select','1,1',null,3,null,null),
(3,12,'check',null,'check','s11.3',null,0,'true',25),
(3,13,'out',null,'output','','Examine subject not found!',0,null,null),
(3,14,'jump',null,'jump','',null,0,null,1000),
(3,15,'out',null,'output','s1.3,s1.1','SELECT o.object_description FROM object o, object_properties op, player p WHERE o.object_id = op.object_id AND LOWER(op.object_id) = LOWER(''?'') AND (op.object_in_inventory = ''true'' OR op.object_in_room = p.player_in_room);' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(3,16,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(3,17,'out',null,'output','s3.3,s3.1','SELECT r.room_description FROM room r, player p WHERE r.room_id = p.player_in_room and Lower(r.room_id) = Lower(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(3,18,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(3,19,'out',null,'output','s5.3,s5.1','SELECT pa.path_description FROM player p, path pa WHERE p.player_in_room = pa.path_from_room AND LOWER(pa.path_direction) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(3,20,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(3,21,'out',null,'output','s7.3,s7.1','SELECT n.npc_description FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND LOWER(n.npc_id) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(3,22,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(3,23,'out',null,'output','s9.3,s9.1','SELECT e.enemy_description FROM enemy e, player p WHERE e.enemy_in_room = p.player_in_room AND LOWER(e.enemy_id) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(3,24,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(3,25,'out',null,'output','s11.1,s11.1,s11.2','GET DESCRIPTION: SELECT ri.inventory_description FROM room_inventory ri, player p WHERE ri.inventory_in_room = p.player_in_room AND LOWER(ri.inventory_id) = LOWER(''?'');' || Char(13) + Char(10) + 'SEARCH FOR OBJECTS: UPDATE room_inv_examine SET inventory_is_examined = ''true'' WHERE LOWER(inventory_id) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(3,26,'update room_inv_examine set inventory_is_examined = ''true'' where Lower(inventory_id) = Lower(?);', 'pcon','update','s11.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(4,'take','adventure','UPDATE object_properties SET object_in_container = null, object_in_room = null, object_in_inventory = ''true'' WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op, object o WHERE o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','take,object',1,'''take <object_name or object_id>'' / ''take object <object_name or object_id>'' Pick up an object near you and store it in your inventory.','english','objectID or objectName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(4,1,'select op.object_id, ''true'' from object_properties op, object o where o.object_id = op.object_id and (Lower(o.object_name) = Lower(?) or Lower(o.object_id) = Lower(?)) and (op.object_in_room is not null or op.object_in_container is not null)','pcon','select','1,1',null,2,null,null),
(4,2,'check',null,'check','s1.2',null,0,'true',5),
(4,3,'out',null,'output','1','Object ? does not exist, has not yet been found or is already in your inventory.',0,null,null),
(4,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(4,5,'UPDATE object_properties SET object_in_container = null, object_in_room = null, object_in_inventory = ''true'' WHERE Lower(object_id) = Lower(?);','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(5,'drop','adventure','UPDATE object_properties SET object_in_inventory = ''false'', object_in_room = (SELECT player_in_room FROM player) WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op, object o WHERE op.object_in_inventory = ''true'' AND LOWER(o.object_name) = LOWER(?) AND o.object_id = op.object_id);','drop,object',1,'''drop <object_name or object_id>'' / ''drop object <object_name or object_id>'' Lay down an object from your inventory.','english','objectID or objectName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(5,1,'select o.object_id, ''true'' from object_properties op, object o where o.object_id = op.object_id and (Lower(o.object_name) = Lower(?) or Lower(o.object_id) = Lower(?)) and op.object_in_inventory = ''true'' ','pcon','select','1,1',null,2,null,null),
(5,2,'check',null,'check','s1.2',null,0,'true',5),
(5,3,'out',null,'output','1','Object ? isn´t in your inventory.',0,null,null),
(5,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(5,5,'UPDATE object_properties SET object_in_inventory = ''false'', object_in_room = (SELECT player_in_room FROM player) WHERE Lower(object_id) = Lower(?) and object_in_inventory = ''true'';','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(6,'give','adventure','UPDATE object_properties SET object_in_inventory = ''false'', object_from_npc = (SELECT n.npc_id FROM npc n WHERE LOWER(n.npc_name) = LOWER(?)) WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op, object o WHERE op.object_in_inventory = ''true'' AND o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','give,npc,object',2,'''give <npc_name or npc_id>,<object_name or object_id>''/ ''give npc <npc_name or npc_id> object <object_name or object_id>'' Hand an object from your inventory to a npc. WARNING! You may not be able to get the object back afterwards. Objects given to traders are sold.','english','npcID or npcName,objectID or objectName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(6,1,'select o.object_id, n.npc_id, ''true'' from object_properties op, object o, npc n where o.object_id = op.object_id and (Lower(o.object_name) = Lower(?) or Lower(o.object_id) = Lower(?)) and op.object_in_inventory = ''true'' and (Lower(n.npc_name) = Lower(?) or Lower(n.npc_id) = Lower(?));','pcon','select','2,2,1,1',null,3,null,null),
(6,2,'check',null,'check','s1.3',null,0,'true',5),
(6,3,'out',null,'output','2,1','Object ? is not in your inventory or NPC ? not exist.',0,null,null),
(6,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(6,5,'UPDATE object_properties SET object_in_inventory = ''false'', object_from_npc = ? WHERE object_id = ? and object_in_inventory = ''true'';','pcon','update','s1.2,s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(7,'use','adventure','UPDATE object_properties SET object_magical_effect_used = ''true'' WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op,  object o WHERE op.object_in_inventory = ''true'' AND op.object_magical_effect_used = ''false'' AND o.object_magical = ''true'' AND o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','use,object',1,'''use <object_name or object_id>'' / ''use object <object_name or object_id>'' Use an object and see what happens.','english','objectID or objectName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(7,1,'select o.object_id, ''true'' from object o, object_properties op where op.object_id = o.object_id and (Lower(o.object_id) = Lower(?) or Lower(o.object_name) = Lower(?)) and op.object_in_inventory = ''true'';','pcon','select','1,1',null,2,null,null),
(7,2,'check',null,'check','s1.2',null,0,'true',5),
(7,3,'out',null,'output','1','Object ? isn´t in your inventory.',0,null,null),
(7,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(7,5,'UPDATE object_properties SET object_magical_effect_used = ''true'' WHERE Lower(object_id) = Lower(?);','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(8,'eat','adventure','UPDATE object_properties SET object_eaten = ''true'' WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op, object o WHERE op.object_in_inventory = ''true'' AND op.object_eaten = ''false'' AND o.object_eatable = ''true'' AND o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','eat,object',1,'''eat <object_name or object_id>'' / ''eat object <object_name or object_id>'' Consume an object and see what happens. WARNING! Object may not be useable again.','english','objectID or objectName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(8,1,'select o.object_id, ''true'' from object o, object_properties op where op.object_id = o.object_id and (Lower(o.object_id) = Lower(?) or Lower(o.object_name) = Lower(?)) and op.object_in_inventory = ''true'';','pcon','select','1,1',null,2,null,null),
(8,2,'check',null,'check','s1.2',null,0,'true',5),
(8,3,'out',null,'output','1','Object ? isn´t in your inventory.',0,null,null),
(8,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(8,5,'UPDATE object_properties SET object_eaten = ''true'' WHERE Lower(object_id) = Lower(?);','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(9,'talk to','adventure','GET TEXT: SELECT n.npc_text FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND LOWER(n.npc_name) = LOWER(?);' || Char(13) + Char(10) || 'RECEIVE QUEST: UPDATE quest_update SET quest_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);' || Char(13) + Char(10) || 'RECEIVE QUEST REWARD: UPDATE quest_update SET quest_reward_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);','talk to,npc,enemy',1,'''talk to <npc_name or npc_id or enemy_name or enemy_id>''/ ''talk to npc <npc_name or npc_id>''/ ''talk to enemy <enemy_name or enemy_id>'' Speak to an npc/enemy. Returns the text of the npc/enemy.','english','npcID or npcName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(9,1,'SELECT n.npc_text, ''true'', n.npc_name, n.npc_id FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND (LOWER(n.npc_name) = LOWER(?) or LOWER(n.npc_id) = LOWER(?));','pcon','select','1,1',null,4,null,null),
(9,2,'check',null,'check','s1.2',null,0,'true',7),
(9,3,'SELECT e.enemy_text, ''true'' FROM enemy e, player p WHERE e.enemy_in_room = p.player_in_room AND (LOWER(e.enemy_name) = LOWER(?) or LOWER(e.enemy_id) = LOWER(?));','pcon','select','1,1',null,2,null,null),
(9,4,'check',null,'check','s3.2',null,0,'true',17),
(9,5,'out',null,'output','1','You are not in the same room as the NPC or Enemy ?.',0,null,null),
(9,6,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(9,7,'out',null,'output','s1.1',CHAR(13) || CHAR(10) || '?',0,null,null),
(9,8,'SELECT ''true'' FROM quest_update WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?) and quest_received = ''false'';','pcon','select','s1.4',null,1,null,null),
(9,9,'check',null,'check','s8.1',null,0,'true',11),
(9,10,'jump',null,'jump','',null,0,null,12),   -- Jump NEXT
(9,11,'UPDATE quest_update SET quest_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);','pcon','update','s1.4',null,0,null,null),
(9,12,'SELECT ''true'' FROM quest_update WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?) and quest_received = ''true'' and quest_solved = ''true'' and quest_reward_received = ''false'';','pcon','select','s1.4',null,1,null,null),
(9,13,'check',null,'check','s12.1',null,0,'true',15),
(9,14,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(9,15,'UPDATE quest_update SET quest_reward_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);','pcon','update','s1.4',null,0,null,null),
(9,16,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(9,17,'out',null,'output','s3.1',CHAR(13) || CHAR(10) || '?',0,null,null);

INSERT INTO ACTIONS VALUES(10,'attack','adventure','INSERT INTO enemy_history values (?,?);','attack,with,enemy,object' ,2,'''attack <enemy_name or enemy_id>''/ ''attack <enemy_name or enemy_id> with <object_name>''/ ''attack <enemy_name or enemy_id>,<object_name or object_id>''/ ''attack enemy <enemy_name or enemy_id> object <object_name or object_id>'' Start fighting an enemy. An object may increase your attack.','english','enemyID or enemyName,objectID or objectName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(10,1,'select enemy_id, ''true'' from enemy where Lower(enemy_name) = Lower(?) or Lower(enemy_id) = Lower(?)','pcon','select','1,1',null,2,null,null),
(10,2,'select object_id from object where Lower(object_name) = Lower(?) or Lower(object_id) = Lower(?)','pcon','select','2,2',null,1,null,null),
(10,3,'check',null,'check','s1.2',null,0,'true',6),
(10,4,'out',null,'output','1','The enemy ? does not exists.',0,null,null),
(10,5,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(10,6,'check',null,'check','2',null,0,'',9),
(10,7,'INSERT INTO enemy_history values (?,?);','pcon','insert','s1.1,s2.1',null,0,null,null),
(10,8,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(10,9,'INSERT INTO enemy_history values (?,null);','pcon','insert','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(11,'inv','adventure','SELECT o.object_name FROM object o, object_properties op WHERE op.object_in_inventory = ''true'' AND o.object_id = op.object_id;','',0,'Returns a list with names of the objects in your inventory.','english',null,'true');
INSERT INTO ACTIONS VALUES(12,'history','adventure','SELECT command_given, command_as_action FROM history ORDER BY command_id desc LIMIT 10;','',0,'Returns your last 10 actions that changed the database.','english',null,'true');
INSERT INTO ACTIONS VALUES(13,'alltables','adventure','SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = ''PUBLIC'' AND TABLE_TYPE = ''VIEW''','',0,'Returns the name of all usable tables\views.','english',null,'true');

INSERT INTO ACTIONS VALUES(14,'trade','adventure','Update object_properties SET object_from_npc = null, object_in_inventory = ''true'' where object_id = any(select object_id from object o where Lower(o.object_name) = Lower(?));','trade,from,object,npc',2,'''trade <object_name or object_id> from <npc_name or npc_id>'' / ''trade <object_name or object_id>,<npc_name or npc_id>'' / ''trade object <object_name or object_id> from npc <npc_name or npc_id>'' Buy objects from a npc that is a merchant.','english','objectID or objectName,npcID or npcName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(14,1,'select npc_id, npc_is_merchant from npc where Lower(npc_id) = Lower(?) or Lower(npc_name) = Lower(?);','pcon','select','2,2',null,2,null,null),
(14,2,'check',null,'check','s1.2',null,0,'true',5),
(14,3,'out',null,'output','2','The NPC ? does not exist,is not a merchant or not found yet.',0,null,null),
(14,4,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(14,5,'select op.object_id, ''true'' from object o, object_properties op, npc n where Lower(n.npc_id) = Lower(?) and n.npc_id = op.object_from_npc and op.object_id = o.object_id and (Lower(o.object_id) = Lower(?) or Lower(o.object_name) = Lower(?));','pcon','select','s1.1,1,1',null,2,null,null),
(14,6,'check',null,'check','s5.2',null,0,'true',9),
(14,7,'out',null,'output','2,1','The NPC ? does not sell the object ?. You can use the viewoffers command to display all offers from a merchant.',0,null,null),
(14,8,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(14,9,'Update object_properties SET object_from_npc = null, object_in_inventory = ''true'' WHERE object_id = ?;','pcon','update','s5.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(15,'viewoffers','adventure','select o.object_id, o.object_name, op.object_value from npc n, object_properties op, object o where op.object_id = o.object_id and Lower(n.npc_name) = Lower(?) and op.object_from_npc = n.npc_id;','viewoffers,npc',1,'''viewoffers <npc_name or npc_id>'' / ''viewoffers npc <npc_name or npc_id>'' Look at the merchant''s offers.','english','npcID or npcName',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(15,1,'select npc_id, npc_is_merchant from npc where Lower(npc_id) = Lower(?) or Lower(npc_name) = Lower(?);','pcon','select','1,1',null,2,null,null),
(15,2,'check',null,'check','s1.2',null,0,'true',5),
(15,3,'out',null,'output','1','The NPC ? does not exist or is not a merchant.',0,null,null),
(15,4,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(15,5,'out',null,'output','1','Merchant offers:',0,null,null),
(15,6,'select o.object_id, o.object_name, op.object_value from npc n, object_properties op, object o where op.object_id = o.object_id and Lower(n.npc_id) = Lower(?) and op.object_from_npc = n.npc_id;','pcon','select out','s1.1',
'
ID: ?, Name: ?, Price: ?',3,null,null);

INSERT INTO ACTIONS VALUES(16,'tablestyle','gui','Update player set player_table_style = ?;','tablestyle',1,'''tablestyle <style_number>'' | 1: table_style, 2: row_style | Change the table style of SELECT query results.','english','tablestyleID',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(16,1,'Update player set player_table_style = ?;','pcon','update','1',null,0,null,null);

INSERT INTO ACTIONS VALUES(17,'fontsize','gui','Update player set player_font_size = ?;','fontsize',1,'''fontsize <size>'' Set the font size of the input und output area.','english','font size',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(17,1,'Update player set player_font_size = ?;','pcon','update','1',null,0,null,null);

INSERT INTO ACTIONS VALUES(18,'clear','gui','clear command','clear',0,'Delete all text from the output area.','english',null,default);

INSERT INTO ACTIONS VALUES(19,'white mode','gui','Update player set player_theme_style = 1;','white mode',0,'Change the theme of the game to the color white.','english',null,default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(19,1,'Update player set player_theme_style = 1;','pcon','update','',null,0,null,null);

INSERT INTO ACTIONS VALUES(20,'dark mode','gui','Update player set player_theme_style = 2;','dark mode',0,'Change the theme of the game to the color black.','english',null,default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(20,1,'Update player set player_theme_style = 2;','pcon','update','',null,0,null,null);

INSERT INTO ACTIONS VALUES(21,'cheat','adventure','?','cheat',1,'''cheat <SQL>'' Execute one SELECT query on hidden tables from the DB to get more information about the game.','english','SELECT SQL','true');
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(21,1,'select ''true'' from object_properties where object_id = ''o7'' and object_in_inventory = ''true'' and object_cheats_left > 0;','pcon','select','',null,1,null,null),
(21,2,'check',null,'check','s1.1',null,0,'true',5),
(21,3,'out',null,'output','1','You can''t cheat because the corresponding cheat object isn''t in your inventory. Try if you can find it somewhere.',0,null,null),
(21,4,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(21,5,'Update objects_properties set object_cheats_left = object_cheats_left - 1 where object_id = ''o7'' and player_id = ?;','gcon','update','p',null,0,null,null),
(21,6,'select ''GRANT CHEATER TO "''|| player_id ||''"'' from players where player_id = ?;','gcon','select execute','p','?',1,null,null);

-- action commands german
INSERT INTO ACTIONS VALUES(22,'help','adventure','SELECT action, action_description FROM actions_deutsch ORDER BY action_id asc;','',0,'Gibt die Befehle des Spiels und eine Beschreibung ihrer Verwendung zurück.','deutsch',null,'true');

INSERT INTO ACTIONS VALUES(23,'geh','adventure','UPDATE player SET player_in_room = (SELECT pa.path_to_room FROM path pa, player p WHERE p.player_in_room = pa.path_from_room AND LOWER(pa.path_direction) = LOWER(?));'
                              ,'geh,nach',1,
                           '''geh (nach) <path_direction>''/ ''geh <path_direction>'' Folge einem Weg zu einem benachbarten Ort. Aendert den players.player_in_room Wert. Moegliche Richtungen sind Norden: ''n'', Süden: ''s'', Westen: ''w'', Osten: ''e'', und Hoch: ''u'', Runter: ''d''.','deutsch','Himmelsrichtung',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(23,1,'UPDATE player SET player_in_room = (SELECT pa.path_to_room FROM path pa, player p WHERE p.player_in_room = pa.path_from_room AND LOWER(?) like LOWER(pa.path_direction + ''%''));','pcon','update','1',null,0,null,null);

INSERT INTO ACTIONS VALUES(24,'untersuche','adventure','','untersuche',1,
                           '''untersuche <subject_name>''\''untersuche <subject_id>'' Gibt die Beschreibung, des untersuchten Objekte, NPC, Raum, Pfad oder Gegner zurück.','deutsch','ID oder Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(24,1,'SELECT TOP 1 o.object_description, ''true'', object_id
        FROM object o, object_properties op, player p
        WHERE o.object_id = op.object_id AND (LOWER(o.object_name) = LOWER(?) OR LOWER(op.object_id) = LOWER(?)) AND (op.object_in_inventory = ''true'' OR op.object_in_room = p.player_in_room);','pcon','select','1,1',null,3,null,null),
(24,2,'check',null,'check','s1.2',null,0,'true',15),
(24,3,'SELECT r.room_description, ''true'', r.room_id FROM room r, player p WHERE r.room_id = p.player_in_room and (Lower(r.room_id) = Lower(?) or Lower(r.room_name) = Lower(?));','pcon','select','1,1',null,3,null,null),
(24,4,'check',null,'check','s3.2',null,0,'true',17),
(24,5,'SELECT pa.path_description, ''true'', pa.path_direction FROM player p, path pa WHERE p.player_in_room = pa.path_from_room AND LOWER(pa.path_direction) = LOWER(?);','pcon','select','1',null,3,null,null),
(24,6,'check',null,'check','s5.2',null,0,'true',19),
(24,7,'SELECT n.npc_description, ''true'', n.npc_id FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND (LOWER(n.npc_name) = LOWER(?) or LOWER(n.npc_id) = LOWER(?));','pcon','select','1,1',null,3,null,null),
(24,8,'check',null,'check','s7.2',null,0,'true',21),
(24,9,'SELECT e.enemy_description, ''true'', e.enemy_id FROM enemy e, player p WHERE e.enemy_in_room = p.player_in_room AND (LOWER(e.enemy_name) = LOWER(?) or LOWER(e.enemy_id) = LOWER(?));','pcon','select','1,1',null,3,null,null),
(24,10,'check',null,'check','s9.2',null,0,'true',23),
(24,11,'SELECT ri.inventory_id ,ri.inventory_description, ''true'' FROM room_inventory ri, player p WHERE ri.inventory_in_room = p.player_in_room AND (LOWER(ri.inventory_name) = LOWER(?) or LOWER(ri.inventory_id) = LOWER(?));','pcon','select','1,1',null,3,null,null),
(24,12,'check',null,'check','s11.3',null,0,'true',25),
(24,13,'out',null,'output','','Examine subject not found!',0,null,null),
(24,14,'jump',null,'jump','',null,0,null,1000),
(24,15,'out',null,'output','s1.3,s1.1','SELECT o.object_description FROM object o, object_properties op, player p WHERE o.object_id = op.object_id AND LOWER(op.object_id) = LOWER(''?'') AND (op.object_in_inventory = ''true'' OR op.object_in_room = p.player_in_room);' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(24,16,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(24,17,'out',null,'output','s3.3,s3.1','SELECT r.room_description FROM room r, player p WHERE r.room_id = p.player_in_room and Lower(r.room_id) = Lower(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(24,18,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(24,19,'out',null,'output','s5.3,s5.1','SELECT pa.path_description FROM player p, path pa WHERE p.player_in_room = pa.path_from_room AND LOWER(pa.path_direction) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(24,20,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(24,21,'out',null,'output','s7.3,s7.1','SELECT n.npc_description FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND LOWER(n.npc_id) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(24,22,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(24,23,'out',null,'output','s9.3,s9.1','SELECT e.enemy_description FROM enemy e, player p WHERE e.enemy_in_room = p.player_in_room AND LOWER(e.enemy_id) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(24,24,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(24,25,'out',null,'output','s11.1,s11.1,s11.2','GET DESCRIPTION: SELECT ri.inventory_description FROM room_inventory ri, player p WHERE ri.inventory_in_room = p.player_in_room AND LOWER(ri.inventory_id) = LOWER(''?'');' || Char(13) + Char(10) + 'SEARCH FOR OBJECTS: UPDATE room_inv_examine SET inventory_is_examined = ''true'' WHERE LOWER(inventory_id) = LOWER(''?'');' || Char(13) + Char(10) + Char(13) + Char(10) || '?',0,null,null),
(24,26,'update room_inv_examine set inventory_is_examined = ''true'' where Lower(inventory_id) = Lower(?);', 'pcon','update','s11.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(25,'nimm','adventure','UPDATE object_properties SET object_in_container = null, object_in_room = null, object_in_inventory = ''true'' WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op, object o WHERE o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','nimm,objekt',1,'''nimm <object_name oder object_id>''  / ''nimm Objekt <object_name oder object_id>'' Nimm einen Gegenstand in deiner Nähe und lagere ihn in deinem Inventar.','deutsch','Objekt ID oder Objekt Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(25,1,'select op.object_id, ''true'' from object_properties op, object o where o.object_id = op.object_id and (Lower(o.object_name) = Lower(?) or Lower(o.object_id) = Lower(?)) and (op.object_in_room is not null or op.object_in_container is not null)','pcon','select','1,1',null,2,null,null),
(25,2,'check',null,'check','s1.2',null,0,'true',5),
(25,3,'out',null,'output','1','Object ? does not exist, has not yet been found or is already in your inventory.',0,null,null),
(25,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(25,5,'UPDATE object_properties SET object_in_container = null, object_in_room = null, object_in_inventory = ''true'' WHERE Lower(object_id) = Lower(?);','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(26,'platziere','adventure','UPDATE object_properties SET object_in_inventory = ''false'', object_in_room = (SELECT player_in_room FROM player) WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op, object o WHERE op.object_in_inventory = ''true'' AND LOWER(o.object_name) = LOWER(?) AND o.object_id = op.object_id);','platziere,objekt',1,'''platziere <object_name oder object_id>'' / ''platziere Objekt <object_name oder object_id>'' Legen Sie einen Gegenstand aus Ihrem Inventar ab.','deutsch','Objekt ID oder Objekt Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(26,1,'select o.object_id, ''true'' from object_properties op, object o where o.object_id = op.object_id and (Lower(o.object_name) = Lower(?) or Lower(o.object_id) = Lower(?)) and op.object_in_inventory = ''true'' ','pcon','select','1,1',null,2,null,null),
(26,2,'check',null,'check','s1.2',null,0,'true',5),
(26,3,'out',null,'output','1','Object ? isn´t in your inventory.',0,null,null),
(26,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(26,5,'UPDATE object_properties SET object_in_inventory = ''false'', object_in_room = (SELECT player_in_room FROM player) WHERE Lower(object_id) = Lower(?) and object_in_inventory = ''true'';','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(27,'gib','adventure','UPDATE object_properties SET object_in_inventory = ''false'', object_from_npc = (SELECT n.npc_id FROM npc n WHERE LOWER(n.npc_name) = LOWER(?)) WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op, object o WHERE op.object_in_inventory = ''true'' AND o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','gib,npc,objekt',2,'''gib <npc_name oder npc_id>,<object_name oder object_id>''/ ''gib npc <npc_name or npc_id> Objekt <object_name or object_id>'' Gib einem NPC einen Gegenstand aus deinem Inventar. WARNUNG! Möglicherweise können Sie das Objekt danach nicht mehr zurückbekommen. Gegenstände, die man an Händler gibt, werden verkauft.','deutsch','Npc ID or Npc Name,Objekt ID or Objekt Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(27,1,'select o.object_id, n.npc_id, ''true'' from object_properties op, object o, npc n where o.object_id = op.object_id and (Lower(o.object_name) = Lower(?) or Lower(o.object_id) = Lower(?)) and op.object_in_inventory = ''true'' and (Lower(n.npc_name) = Lower(?) or Lower(n.npc_id) = Lower(?));','pcon','select','2,2,1,1',null,3,null,null),
(27,2,'check',null,'check','s1.3',null,0,'true',5),
(27,3,'out',null,'output','2,1','Object ? is not in your inventory or NPC ? not exist.',0,null,null),
(27,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(27,5,'UPDATE object_properties SET object_in_inventory = ''false'', object_from_npc = ? WHERE object_id = ? and object_in_inventory = ''true'';','pcon','update','s1.2,s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(28,'nutze','adventure','UPDATE object_properties SET object_magical_effect_used = ''true'' WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op,  object o WHERE op.object_in_inventory = ''true'' AND op.object_magical_effect_used = ''false'' AND o.object_magical = ''true'' AND o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','nutze,objekt',1,'''nutze <object_name oder object_id>'' / ''nutze Objekt <object_name oder object_id>'' Verwenden Sie ein Objekt und sehen Sie, was passiert.','deutsch','Objekt ID oder Objekt Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(28,1,'select o.object_id, ''true'' from object o, object_properties op where op.object_id = o.object_id and (Lower(o.object_id) = Lower(?) or Lower(o.object_name) = Lower(?)) and op.object_in_inventory = ''true'';','pcon','select','1,1',null,2,null,null),
(28,2,'check',null,'check','s1.2',null,0,'true',5),
(28,3,'out',null,'output','1','Object ? isn´t in your inventory.',0,null,null),
(28,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(28,5,'UPDATE object_properties SET object_magical_effect_used = ''true'' WHERE Lower(object_id) = Lower(?);','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(29,'iss','adventure','UPDATE object_properties SET object_eaten = ''true'' WHERE object_id = (SELECT TOP 1 op.object_id FROM object_properties op,  object o WHERE op.object_in_inventory = ''true'' AND op.object_eaten = ''false'' AND o.object_eatable = ''true'' AND o.object_id = op.object_id AND LOWER(o.object_name) = LOWER(?));','iss,objekt',1,'''iss <object_name oder object_id>'' / ''iss Objekt <object_name oder object_id>'' Konsumieren Sie ein Objekt und sehen Sie, was passiert. WARNUNG! Das Objekt ist möglicherweise nicht mehr verwendbar.','deutsch','Objekt ID or Objekt Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(29,1,'select o.object_id, ''true'' from object o, object_properties op where op.object_id = o.object_id and (Lower(o.object_id) = Lower(?) or Lower(o.object_name) = Lower(?)) and op.object_in_inventory = ''true'';','pcon','select','1,1',null,2,null,null),
(29,2,'check',null,'check','s1.2',null,0,'true',5),
(29,3,'out',null,'output','1','Object ? isn´t in your inventory.',0,null,null),
(29,4,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(29,5,'UPDATE object_properties SET object_eaten = ''true'' WHERE Lower(object_id) = Lower(?);','pcon','update','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(30,'sprich zu','adventure','GET TEXT: SELECT n.npc_text FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND LOWER(n.npc_name) = LOWER(?);' || Char(13) + Char(10) || 'RECEIVE QUEST: UPDATE quest_update SET quest_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);' || Char(13) + Char(10) || 'RECEIVE QUEST REWARD: UPDATE quest_update SET quest_reward_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);','sprich zu,npc,gegner',1,'''sprich zu <npc_name oder npc_id oder enemy_name oder enemy_id>''/ ''sprich zu Npc <npc_name oder npc_id>''/ ''sprich zu gegner <enemy_name oder enemy_id>'' Sprich mit einem NPC/Gegner. Gibt den Text des NPCs/Gegners zurück.','deutsch','Npc ID or Npc Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(30,1,'SELECT n.npc_text, ''true'', n.npc_name, n.npc_id FROM npc n, player p WHERE n.npc_in_room = p.player_in_room AND (LOWER(n.npc_name) = LOWER(?) or LOWER(n.npc_id) = LOWER(?));','pcon','select','1,1',null,4,null,null),
(30,2,'check',null,'check','s1.2',null,0,'true',7),
(30,3,'SELECT e.enemy_text, ''true'' FROM enemy e, player p WHERE e.enemy_in_room = p.player_in_room AND (LOWER(e.enemy_name) = LOWER(?) or LOWER(e.enemy_id) = LOWER(?));','pcon','select','1,1',null,2,null,null),
(30,4,'check',null,'check','s3.2',null,0,'true',17),
(30,5,'out',null,'output','1','You are not in the same room as the NPC or Enemy ?.',0,null,null),
(30,6,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(30,7,'out',null,'output','s1.1',CHAR(13) || CHAR(10) || '?',0,null,null),
(30,8,'SELECT ''true'' FROM quest_update WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?) and quest_received = ''false'';','pcon','select','s1.4',null,1,null,null),
(30,9,'check',null,'check','s8.1',null,0,'true',11),
(30,10,'jump',null,'jump','',null,0,null,12),   -- Jump NEXT
(30,11,'UPDATE quest_update SET quest_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);','pcon','update','s1.4',null,0,null,null),
(30,12,'SELECT ''true'' FROM quest_update WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?) and quest_received = ''true'' and quest_solved = ''true'' and quest_reward_received = ''false'';','pcon','select','s1.4',null,1,null,null),
(30,13,'check',null,'check','s12.1',null,0,'true',15),
(30,14,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(30,15,'UPDATE quest_update SET quest_reward_received = ''true'' WHERE quest_id = any(select quest_id from quest q,npc n where n.npc_id = q.quest_from_npc and n.npc_id = ?);','pcon','update','s1.4',null,0,null,null),
(30,16,'jump',null,'jump','',null,0,null,1000),   -- Jump End
(30,17,'out',null,'output','s3.1',CHAR(13) || CHAR(10) || '?',0,null,null);

INSERT INTO ACTIONS VALUES(31,'angreifen','adventure','INSERT INTO enemy_history values (?,?);','angreifen,mit,gegner,objekt' ,2,'''angreifen <enemy_name oder enemy_id>''/ ''angreifen <enemy_name oder enemy_id> mit <object_name>''/ ''angreifen <enemy_name oder enemy_id>,<object_name oder object_id>''/ ''angreifen Gegner <enemy_name oder enemy_id> Objekt <object_name oder object_id>'' Beginnen Sie mit dem Kampf gegen einen Gegner. Ein Gegenstand kann Ihren Angriff verstärken.','deutsch','Gegner ID oder Gegner Name,Objekt ID or Objekt Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(31,1,'select enemy_id, ''true'' from enemy where Lower(enemy_name) = Lower(?) or Lower(enemy_id) = Lower(?)','pcon','select','1,1',null,2,null,null),
(31,2,'select object_id from object where Lower(object_name) = Lower(?) or Lower(object_id) = Lower(?)','pcon','select','2,2',null,1,null,null),
(31,3,'check',null,'check','s1.2',null,0,'true',6),
(31,4,'out',null,'output','1','The enemy ? does not exists.',0,null,null),
(31,5,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(31,6,'check',null,'check','2',null,0,'',9),
(31,7,'INSERT INTO enemy_history values (?,?);','pcon','insert','s1.1,s2.1',null,0,null,null),
(31,8,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(31,9,'INSERT INTO enemy_history values (?,null);','pcon','insert','s1.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(32,'inv','adventure','SELECT o.object_name FROM object o, object_properties op WHERE op.object_in_inventory = ''true'' AND o.object_id = op.object_id;','',0,'Gibt eine Liste mit Namen der Objekte in Ihrem Inventar zurück.','deutsch',null,'true');
INSERT INTO ACTIONS VALUES(33,'history','adventure','SELECT command_given, command_as_action FROM history ORDER BY command_id desc LIMIT 10;','',0,'Gibt Ihre letzten 10 Aktionen zurück.','deutsch',null,'true');
INSERT INTO ACTIONS VALUES(34,'alltables','adventure','SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = ''PUBLIC'' AND TABLE_TYPE = ''VIEW''','',0,'Gibt den Namen aller benutzbaren Tabellen\Views zurück.','deutsch',null,'true');

INSERT INTO ACTIONS VALUES(35,'handeln','adventure','Update object_properties SET object_from_npc = null, object_in_inventory = ''true'' where object_id = any(select object_id from object o where Lower(o.object_name) = Lower(?));','handeln,von,objekt,npc',2,'''handeln <object_name or object_id> von <npc_name or npc_id>'' / ''handeln <object_name or object_id>,<npc_name or npc_id>'' / ''handeln Objekt <object_name or object_id> von Npc <npc_name or npc_id>'' Kaufe Objekte von einem NPC, der ein Händler ist.','deutsch','Objekt ID or Objekt Name,Npc ID or Npc Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(35,1,'select npc_id, npc_is_merchant from npc where Lower(npc_id) = Lower(?) or Lower(npc_name) = Lower(?);','pcon','select','2,2',null,2,null,null),
(35,2,'check',null,'check','s1.2',null,0,'true',5),
(35,3,'out',null,'output','2','The NPC ? does not exist or is not a merchant.',0,null,null),
(35,4,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(35,5,'select op.object_id, ''true'' from object o, object_properties op, npc n where Lower(n.npc_id) = Lower(?) and n.npc_id = op.object_from_npc and op.object_id = o.object_id and (Lower(o.object_id) = Lower(?) or Lower(o.object_name) = Lower(?));','pcon','select','s1.1,1,1',null,2,null,null),
(35,6,'check',null,'check','s5.2',null,0,'true',9),
(35,7,'out',null,'output','2,1','The NPC ? does not sell the object ?. You can use the viewoffers command to display all offers from a merchant.',0,null,null),
(35,8,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(35,9,'Update object_properties SET object_from_npc = null, object_in_inventory = ''true'' WHERE object_id = ?;','pcon','update','s5.1',null,0,null,null);

INSERT INTO ACTIONS VALUES(36,'zeige Angebot','adventure','select o.object_id, o.object_name, o.object_value from npc n, object_properties op, object o where op.object_id = o.object_id and Lower(n.npc_name) = Lower(?) and op.object_from_npc = n.npc_id;','zeige angebot,npc',1,'''zeige Angebot <npc_name or npc_id>'' / ''zeige Angebot Npc <npc_name or npc_id>'' Schauen Sie sich die Angebote des Händlers an.','deutsch','Npc ID oder Npc Name',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(36,1,'select npc_id, npc_is_merchant from npc where Lower(npc_id) = Lower(?) or Lower(npc_name) = Lower(?);','pcon','select','1,1',null,2,null,null),
(36,2,'check',null,'check','s1.2',null,0,'true',5),
(36,3,'out',null,'output','1','The NPC ? does not exist or is not a merchant.',0,null,null),
(36,4,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(36,5,'out',null,'output','1','Merchant offers:',0,null,null),
(36,6,'select o.object_id, o.object_name, op.object_value from npc n, object_properties op, object o where op.object_id = o.object_id and Lower(n.npc_id) = Lower(?) and op.object_from_npc = n.npc_id;','pcon','select out','s1.1',
'
ID: ?, Name: ?, Price: ?',3,null,null);

INSERT INTO ACTIONS VALUES(37,'Tabellenstil','gui','Update player set player_table_style = ?;','tabellenstil',1,'''Tabellenstil <stil_nummer>'' | 1: table_style, 2: row_style | Ändert den Tabellenstil von SELECT-Anfrage Ergebnissen.','deutsch','Tabellen Stil ID',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(37,1,'Update player set player_table_style = ?;','pcon','update','1',null,0,null,null);

INSERT INTO ACTIONS VALUES(38,'Schriftgröße','gui','Update player set player_font_size = ?;','schriftgröße',1,'''Schriftgröße <größe>'' Set the font size of the input und output area. ','deutsch','Schriftgröße',default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(38,1,'Update player set player_font_size = ?;','pcon','update','1',null,0,null,null);

INSERT INTO ACTIONS VALUES(39,'leeren','gui','leeren Befehl','leeren',0,'Löschen Sie den gesamten Text aus dem Ausgabebereich.','deutsch',null,default);

INSERT INTO ACTIONS VALUES(40,'heller Modus','gui','Update player set player_theme_style = 1;','heller modus',0,'Ändern Sie das Thema des Spiels in die Farbe weiß.','deutsch',null,default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(40,1,'Update player set player_theme_style = 1;','pcon','update','',null,0,null,null);

INSERT INTO ACTIONS VALUES(41,'dunkler Modus','gui','Update player set player_theme_style = 2;','dunkler modus',0,'Ändern Sie das Thema des Spiels in die Farbe schwarz.','deutsch',null,default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(41,1,'Update player set player_theme_style = 2;','pcon','update','',null,0,null,null);

INSERT INTO ACTIONS VALUES(42,'cheat','adventure','?','cheat',1,'''cheat <SQL>'' Führen Sie eine SELECT-Abfrage für versteckte Tabellen aus der Datenbank aus, um weitere Informationen über das Spiel zu erhalten.','deutsch','SELECT SQL','true');
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(42,1,'select ''true'' from object_properties where object_id = ''o7'' and object_in_inventory = ''true'' and object_cheats_left > 0;','pcon','select','',null,1,null,null),
(42,2,'check',null,'check','s1.1',null,0,'true',5),
(42,3,'out',null,'output','1','You can''t cheat because the corresponding cheat object isn''t in your inventory. Try if you can find it somewhere.',0,null,null),
(42,4,'jump',null,'jump','',null,0,null,1000),   -- Jump END
(42,5,'Update objects_properties set object_cheats_left = object_cheats_left - 1 where object_id = ''o7'' and player_id = ?;','gcon','update','p',null,0,null,null),
(42,6,'select ''GRANT CHEATER TO "''|| player_id ||''"'' from players where player_id = ?;','gcon','select execute','p','?',1,null,null);

-- system command
INSERT INTO ACTIONS VALUES(43,'revoke cheater','adventure','','',0,'','system',null,default);
INSERT INTO ACTIONS_SEQUENCES (action_id,action_sequence_id,action_sql_command,action_connection,action_type,action_need_attribute,
                               action_output,action_select_count,action_check_attribute_with,action_jump_to) VALUES
(43,1,'select ''REVOKE CHEATER FROM "''|| player_id ||''" RESTRICT'' from players where player_id = ?;','gcon','select execute','p','?',1,null,null);