-- initialize new game after new player
CREATE TRIGGER INITIALIZE_GAME_FOR_NEW_PLAYER AFTER INSERT ON PLAYERS
    REFERENCING NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE i INTEGER DEFAULT 0;
    DECLARE j INTEGER DEFAULT 0;
    DECLARE object_ivalue decimal(7,2);
    DECLARE object_icheats integer;
    DECLARE object_iplace varchar(10);
    DECLARE object_ivisible varchar(5);
    DECLARE object_iid varchar(10);
    DECLARE enemy_iid varchar(10);
    DECLARE enemy_ilife_points integer;
    DECLARE container_iid varchar(10);
    DECLARE quest_iid varchar(10);
    DECLARE quest_in_game boolean;
    DECLARE trash_iid varchar(10);
    DECLARE trash_n varchar(50);
    DECLARE trash_room varchar(10);
    DECLARE trash_in_game boolean;


    FOR SELECT * from objects where game_id = newrow.PLAYER_PLAYS_GAME DO
        SET object_iid = select object_id from objects where game_id = newrow.player_plays_game limit 1 offset i;
        SET object_ivalue = select object_initial_value from objects where object_id = object_iid and game_id = newrow.player_plays_game;
        SET object_icheats = select object_useable_cheats from objects where object_id = object_iid and game_id = newrow.player_plays_game;
        SET object_iplace = select object_initial_place from objects where object_id = object_iid and game_id = newrow.player_plays_game;
        SET object_ivisible = select object_initial_visible from objects where object_id = object_iid and game_id = newrow.player_plays_game;

        IF object_iplace like 'c%' THEN
            Insert  into OBJECTS_PROPERTIES (player_id, object_id, object_value, object_eaten, object_switched_on, object_magical_effect_used,
                                             object_cheats_left, object_in_container, object_in_room, object_from_npc, object_from_enemy,
                                             object_in_inventory, object_is_visible) values
            (newrow.player_id,object_iid, object_ivalue, default,default,default,object_icheats,object_iplace,null,null,null,default,object_ivisible);
        ELSEIF object_iplace like 'r%' THEN
            Insert  into OBJECTS_PROPERTIES (player_id, object_id, object_value, object_eaten, object_switched_on, object_magical_effect_used,
                                             object_cheats_left, object_in_container, object_in_room, object_from_npc, object_from_enemy,
                                             object_in_inventory, object_is_visible) values
            (newrow.player_id,object_iid, object_ivalue, default,default,default,object_icheats,null,object_iplace,null,null,default,object_ivisible);
        ELSEIF object_iplace like 'n%' THEN
            Insert  into OBJECTS_PROPERTIES (player_id, object_id, object_value, object_eaten, object_switched_on, object_magical_effect_used,
                                             object_cheats_left, object_in_container, object_in_room, object_from_npc, object_from_enemy,
                                             object_in_inventory, object_is_visible) values
            (newrow.player_id,object_iid, object_ivalue, default,default,default,object_icheats,null,null,object_iplace,null,default,object_ivisible);
        ELSEIF object_iplace like 'e%' THEN
            Insert  into OBJECTS_PROPERTIES (player_id, object_id, object_value, object_eaten, object_switched_on, object_magical_effect_used,
                                             object_cheats_left, object_in_container, object_in_room, object_from_npc, object_from_enemy,
                                             object_in_inventory, object_is_visible) values
            (newrow.player_id,object_iid, object_ivalue, default,default,default,object_icheats,null,null,null,object_iplace,default,object_ivisible);
        ELSEIF object_iplace like 'inv' THEN
            Insert  into OBJECTS_PROPERTIES (player_id, object_id, object_value, object_eaten, object_switched_on, object_magical_effect_used,
                                             object_cheats_left, object_in_container, object_in_room, object_from_npc, object_from_enemy,
                                             object_in_inventory, object_is_visible) values
            (newrow.player_id,object_iid, object_ivalue, default,default,default,object_icheats,null,null,null,null,'true',object_ivisible);
        END IF;

        SET i = i + 1;
    END FOR;

    FOR SELECT * from enemies where game_id = newrow.PLAYER_PLAYS_GAME DO
        SET enemy_iid = select enemy_id from enemies where game_id = newrow.player_plays_game limit 1 offset j;
        SET enemy_ilife_points = select enemy_life_points from enemies where enemy_id = enemy_iid and game_id = newrow.player_plays_game;

        Insert into enemy_fights (player_id, enemy_id, enemy_current_life_points, enemy_stored_player_money, enemy_defeated) values
        (newrow.player_id,enemy_iid,enemy_ilife_points,default,default);

        SET j = j + 1;
    END FOR;

    SET j = 0;
    FOR SELECT * from containers where container_in_room = any(select room_id from ROOMS where ROOM_IN_GAME = newrow.PLAYER_PLAYS_GAME) DO
        SET container_iid = select container_id from containers where container_in_room = any(select room_id from ROOMS where ROOM_IN_GAME = newrow.PLAYER_PLAYS_GAME) limit 1 offset j;
        INSERT into container_examine values (newrow.player_id, container_iid, default, default);
        SET j = j + 1;
    END FOR;

    SET j = 1;
    FOR SELECT * from quests DO
        SET quest_in_game = exists (SELECT * from quests where QUEST_ID = 'q' || j and QUEST_FROM_NPC = any(select npc_id from npcs where npc_in_room = any(select room_id from ROOMS where ROOM_IN_GAME = newrow.PLAYER_PLAYS_GAME)));
        IF quest_in_game THEN
            SET quest_iid = 'q' || j;
            INSERT into QUESTS_RECEIVED values (newrow.player_id, quest_iid, 'false', 'false', 'false', 'false');
        END IF;
        SET j = j + 1;
    END FOR;

    SET j = 1;
    FOR SELECT * from trashs DO
        SET trash_in_game = exists (SELECT * from trashs where trash_id = 't' || j and trash_in_room = any(select room_id from ROOMS where ROOM_IN_GAME = newrow.PLAYER_PLAYS_GAME));
        IF quest_in_game THEN
            SET trash_iid = 't' || j;
            SET trash_n = select trash from trashs where trash_id = trash_iid;
            SET trash_room = select trash_in_room from trashs where trash_id = trash_iid;
            INSERT into TRASH_REPORT values (newrow.player_id, trash_iid, trash_n, trash_room , default);
        END IF;
        SET j = j + 1;
    END FOR;

    Insert into rooms_visited (player_id, room_id) values (newrow.player_id,newrow.player_in_room);
END;

-- make new objects visible that are in the room (only in the room, this include object,container,quest,trash)
CREATE TRIGGER MAKE_ROOM_OBJECT_VISIBLE AFTER INSERT ON ROOMS_VISITED
    REFERENCING NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    UPDATE objects_properties SET object_is_visible = 'true' WHERE object_in_room = newrow.room_id AND player_id = newrow.player_id;
    UPDATE objects_properties SET object_is_visible = 'true' WHERE object_from_npc =
                                                                (select npc_id from NPCS where newrow.room_id = npc_in_room and npc_is_merchant = 'true')
                                                               AND player_id = newrow.player_id;
    UPDATE CONTAINER_EXAMINE SET CONTAINER_IS_VISIBLE = 'true' WHERE CONTAINER_ID = any(select CONTAINER_ID from CONTAINERS where CONTAINER_IN_ROOM = newrow.ROOM_ID) AND player_id = newrow.player_id;
    UPDATE QUESTS_RECEIVED SET QUEST_VISIBLE = 'true'
                           WHERE QUEST_ID = any(select quest_id from quests
                                            where quest_from_npc = any(select npc_id from npcs where npc_in_room = newrow.ROOM_ID)) and PLAYER_ID = newrow.PLAYER_ID;
    UPDATE trash_report SET trash_is_visible = 'true' where trash_in_room = newrow.room_id and PLAYER_ID = newrow.PLAYER_ID;
END;

-- when the object location changes, checks whether the room is the same.
CREATE TRIGGER CHECK_IF_OBJECT_AND_PLAYER_IN_SAME_ROOM BEFORE UPDATE OF object_in_room,object_in_container,object_from_npc,object_from_enemy ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE player_room varchar(10);
    DECLARE old_npc_room varchar(10);
    DECLARE new_npc_room varchar(10);
    DECLARE old_container_room varchar(10);
    DECLARE new_container_room varchar(10);
    DECLARE container_needs_object boolean;
    DECLARE old_enemy_room varchar(10);
    DECLARE new_enemy_room varchar(10);
    SET player_room = select player_in_room from players where player_id = oldrow.player_id;

	IF (player_room <> oldrow.object_in_room or player_room <> newrow.object_in_room) THEN
        SIGNAL SQLSTATE '45005' SET MESSAGE_TEXT = 'Object isn´t in the same room.';
	END IF;

	SET old_npc_room = select npc_in_room
	               from npcs n, rooms_visited r
	               where n.npc_id = oldrow.object_from_npc
	                and r.player_id = oldrow.player_id
                    and r.room_id = n.npc_in_room;
    SET new_npc_room = select npc_in_room
                   from npcs n, rooms_visited r
                   where n.npc_id = newrow.object_from_npc
                    and r.player_id = oldrow.player_id
                    and r.room_id = n.npc_in_room;
	IF (player_room <> old_npc_room or player_room <> new_npc_room) THEN
	    SIGNAL SQLSTATE '45005' SET MESSAGE_TEXT = 'Object isn´t in the same room.';
	END IF;

    SET old_container_room = select container_in_room
	               from containers c, rooms_visited r
	               where c.container_id = oldrow.object_in_container
	                and r.player_id = oldrow.player_id
                    and r.room_id = c.container_in_room;
	SET new_container_room = select container_in_room
                             from containers c, rooms_visited r
                             where c.container_id = newrow.object_in_container
                               and r.player_id = oldrow.player_id
                               and r.room_id = c.container_in_room;
	IF (player_room <> old_container_room or player_room <> new_container_room) THEN
	    SIGNAL SQLSTATE '45005' SET MESSAGE_TEXT = 'Object isn´t in the same room.';
    END IF;

    SET container_needs_object = exists(select * from CONTAINERS c
                                                 where newrow.OBJECT_ID = c.CONTAINER_NEEDS_OBJECT
                                                    and newrow.OBJECT_IN_CONTAINER = c.CONTAINER_ID);

    IF (container_needs_object) THEN
        SIGNAL SQLSTATE '45005' SET MESSAGE_TEXT = 'The object can not be placed in the container because the container needs the object to examine it closely.';
    END IF;

    SET old_enemy_room = select enemy_in_room
	               from enemies e, rooms_visited r
	               where e.enemy_id = oldrow.object_from_enemy
	                and r.player_id = oldrow.player_id
                    and r.room_id = e.enemy_in_room;
	SET new_enemy_room = select enemy_in_room
                         from enemies e, rooms_visited r
                         where e.enemy_id = newrow.object_from_enemy
                           and r.player_id = oldrow.player_id
                           and r.room_id = e.enemy_in_room;
	IF (player_room <> old_enemy_room or player_room <> new_enemy_room) THEN
	    SIGNAL SQLSTATE '45005' SET MESSAGE_TEXT = 'Object isn´t in the same room.';
    END IF;
END;

-- create new object place output after object location changes
CREATE TRIGGER CHECK_UPDATE_OF_OBJECT_PLACE_AFTER AFTER UPDATE OF object_in_inventory, OBJECT_FROM_NPC, OBJECT_FROM_ENEMY, OBJECT_IN_CONTAINER, OBJECT_IN_ROOM ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE max_output_id integer;
    DECLARE output_inv VARCHAR(200);
    DECLARE object_n varchar(50);
    DECLARE npc_n varchar(50);
    DECLARE enemy_n varchar(50);
    DECLARE container_n varchar(50);

    SET object_n = select object_name from objects o where object_id = newrow.object_id;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    -- drop
    IF (oldrow.OBJECT_IN_ROOM is null and newrow.OBJECT_IN_ROOM is not null) THEN
        SET output_inv = CHAR(13)+CHAR(10) || 'You drop the object ' || object_n || ' in the room ' || newrow.OBJECT_IN_ROOM || '.';
        Insert into output_history values (newrow.PLAYER_ID,max_output_id,output_inv,CURRENT_TIMESTAMP,true);
    ELSEIF (oldrow.OBJECT_IN_CONTAINER is null and newrow.OBJECT_IN_CONTAINER is not null) THEN
        SET container_n = select container_name from containers where container_id = newrow.object_in_container;
        SET output_inv = CHAR(13)+CHAR(10) || 'You drop the object ' || object_n || ' in the room_inventory ' || container_n || '.';
        Insert into output_history values (newrow.player_id,max_output_id,output_inv,CURRENT_TIMESTAMP,true);
    ELSEIF (oldrow.OBJECT_FROM_ENEMY is null and newrow.OBJECT_FROM_ENEMY is not null) THEN
        SET enemy_n = select enemy_name from enemies where enemy_id = newrow.object_from_enemy;
        SET output_inv = CHAR(13)+CHAR(10) || 'You give the object ' || object_n || ' to the enemy ' || enemy_n || '.';
        Insert into output_history values (newrow.player_id,max_output_id,output_inv,CURRENT_TIMESTAMP,true);
    ELSEIF (oldrow.OBJECT_FROM_NPC is null and newrow.OBJECT_FROM_NPC is not null) THEN  -- give
        SET npc_n = select npc_name from npcs where npc_id = newrow.object_from_npc;
        SET output_inv = CHAR(13)+CHAR(10) || 'You give the object ' || object_n || ' to the npc ' || npc_n || '.';
        Insert into output_history values (newrow.player_id,max_output_id,output_inv,CURRENT_TIMESTAMP,true);
    -- take
    ELSEIF (oldrow.OBJECT_IN_INVENTORY = 'false' and newrow.OBJECT_IN_INVENTORY = 'true') THEN
        SET output_inv = CHAR(13)+CHAR(10) || 'You take the object ' || object_n || ' in your inventory.';
        Insert into output_history values (newrow.player_id,max_output_id,output_inv,CURRENT_TIMESTAMP,true);
    end if;
END;

-- check moving to specified room (go)
CREATE TRIGGER CHECK_IF_PATH_TO_NEXT_ROOM_IS_AVAILABLE BEFORE UPDATE OF player_in_room ON PLAYERS
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
	DECLARE room_was_not_visited_before BOOLEAN;
	DECLARE no_path_to_room BOOLEAN;
	SET room_was_not_visited_before = not exists (select * from rooms_visited rv, rooms r where rv.player_id = newrow.player_id AND rv.room_id = newrow.player_in_room
						AND r.room_id = rv.room_id AND r.room_in_game = newrow.player_plays_game);
	SET no_path_to_room = not exists (select * from paths where path_from_room = oldrow.player_in_room AND path_to_room = newrow.player_in_room
                                 		AND (path_needs_object is null OR path_needs_object = any(SELECT o.object_id
										          		  FROM objects_properties o
										                          WHERE newrow.player_id = o.player_id AND o.object_in_inventory = 'true'
													  )
						     )
				   );

	IF (room_was_not_visited_before AND no_path_to_room) THEN
		SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Room not available. Please make sure, there is a path from your current room to this room or if you need an object to enter the room.';
    END IF;
END;

-- add new visited room and (go output)
CREATE TRIGGER INSERT_VISITED_ROOM AFTER UPDATE OF player_in_room ON PLAYERS
    REFERENCING NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE i integer default 0;
    DECLARE max_output_id integer;
    DECLARE room_a VARCHAR(50);
    DECLARE room_n VARCHAR(50);
    DECLARE room_t VARCHAR(500);
    DECLARE room_details VARCHAR(2000);
    DECLARE container_a VARCHAR(50);
    DECLARE container_n VARCHAR(50);
    DECLARE object_a VARCHAR(50);
    DECLARE object_n VARCHAR(50);
    DECLARE npc_a VARCHAR(50);
    DECLARE npc_n VARCHAR(50);
    DECLARE npc_t VARCHAR(100);
    DECLARE enemy_a VARCHAR(50);
    DECLARE enemy_n VARCHAR(50);
    DECLARE enemy_t VARCHAR(100);
    DECLARE path_a VARCHAR(50);
    DECLARE path_t VARCHAR(50);
    DECLARE path_d VARCHAR(10);
    DECLARE room_is_not_dark boolean;

    IF not exists (select * from rooms_visited where player_id = newrow.player_id AND room_id = newrow.player_in_room) THEN
		Insert into rooms_visited values (newrow.player_id,newrow.player_in_room);
    END IF;

    -- output
    SET room_a = select room_article from rooms where newrow.player_in_room = room_id;
    SET room_n = select room_name from rooms where newrow.player_in_room = room_id;
    SET room_t = select room_text from rooms where newrow.player_in_room = room_id;
    SET room_details = CHAR(13)+CHAR(10) || 'You enter ' || room_a || ' ' || room_n || '.' || CHAR(13)+CHAR(10) || room_t;
    SET room_is_not_dark = exists (select * from ROOMS where ROOM_ID = newrow.PLAYER_IN_ROOM
                                                    and (ROOM_IS_DARK = 'false'
                                                    or exists (select * from OBJECTS_PROPERTIES
                                                                        where PLAYER_ID = newrow.PLAYER_ID
                                                                          and OBJECT_SWITCHED_ON = 'true'
                                                                          and OBJECT_IN_INVENTORY = 'true')));

    IF (room_is_not_dark) THEN
        FOR SELECT * from CONTAINERS where CONTAINER_IN_ROOM = newrow.PLAYER_IN_ROOM DO
            SET container_n = select container_name from containers where CONTAINER_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET container_a = select container_article from containers where CONTAINER_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET room_details = room_details || CHAR(13)+CHAR(10) || 'INV: You see ' || container_a || ' ' || container_n || '.';
            SET i = i + 1;
        END FOR;

        SET i = 0;

        FOR SELECT * FROM OBJECTS_PROPERTIES WHERE OBJECT_IN_ROOM = newrow.PLAYER_IN_ROOM AND player_id = newrow.PLAYER_ID DO
            SET object_n = select o.object_name from objects o, objects_properties op where o.object_id = op.object_id and op.OBJECT_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET object_a = select o.object_article from objects o, objects_properties op where o.object_id = op.object_id and op.OBJECT_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET room_details = room_details || CHAR(13)+CHAR(10) || 'OBJECT: You see ' || object_a || ' ' || object_n || '.';
            SET i = i + 1;
        END FOR;

        SET i = 0;

        FOR SELECT * from NPCS where NPC_IN_ROOM = newrow.PLAYER_IN_ROOM DO
            SET npc_n = select npc_name from npcs where NPC_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET npc_a = select npc_article from npcs where NPC_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET npc_t = select npc_type from npcs where NPC_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET room_details = room_details || CHAR(13)+CHAR(10) || 'NPC: You see ' || npc_a || ' ' || npc_t || ' ' || npc_n || '.';
            SET i = i + 1;
        END FOR;

        SET i = 0;

        FOR SELECT * from ENEMIES where ENEMY_IN_ROOM = newrow.PLAYER_IN_ROOM DO
            SET enemy_n = select enemy_name from enemies where ENEMY_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET enemy_a = select enemy_article from enemies where ENEMY_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET enemy_t = select enemy_type from enemies where ENEMY_IN_ROOM = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET room_details = room_details || CHAR(13)+CHAR(10) || 'ENEMY: You see ' || enemy_a || ' ' || enemy_t || ' ' || enemy_n || '.';
            SET i = i + 1;
        END FOR;

        SET i = 0;

        FOR SELECT * from PATHS where PATH_FROM_ROOM = newrow.PLAYER_IN_ROOM DO
            SET path_t = select path_type from paths where path_from_room = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET path_a = select path_article from paths where path_from_room = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET path_d = select path_direction from paths where path_from_room = newrow.PLAYER_IN_ROOM limit 1 offset i;
            SET room_details = room_details || CHAR(13)+CHAR(10) || 'PATH: You see ' || path_a || ' ' || path_t || ' in the direction ' || path_d || '.';
            SET i = i + 1;
        END FOR;
    END IF;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    Insert into output_history values (newrow.player_id,max_output_id,room_details,CURRENT_TIMESTAMP,true);
END;

--- trade Triggers -----
CREATE TRIGGER CHECK_TRADE_MONEY BEFORE UPDATE OF object_from_npc ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
	DECLARE player_money DECIMAL(7,2);
	DECLARE curr_object_value DECIMAL(7,2);
	DECLARE is_merchant BOOLEAN;
	SET player_money = select player_money from players where player_id = oldrow.player_id;
    SET curr_object_value = select object_value from objects_properties where oldrow.object_id = object_id and player_id = oldrow.player_id;
    SET is_merchant = exists (select * from npcs where oldrow.object_from_npc = npc_id and npc_is_merchant = 'true');
	IF (player_money < curr_object_value AND is_merchant) THEN
		SIGNAL SQLSTATE '45001' SET MESSAGE_TEXT = 'Not enough money to buy this Object. Collect more money and come back later when you have enough money.';
    END IF;
END;

CREATE TRIGGER UPDATE_PLAYER_MONEY_AFTER_TRADE AFTER UPDATE OF object_from_npc ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
	DECLARE curr_object_value DECIMAL(7,2);
	DECLARE is_merchant_sell BOOLEAN;
	DECLARE is_merchant_buy BOOLEAN;
    DECLARE max_output_id integer;
    DECLARE trade_output varchar(200) default '';
    DECLARE object_n varchar(50);
    DECLARE npc_n varchar(50);
    DECLARE new_player_money decimal(7,2);

    SET object_n = select object_name from objects where object_id = newrow.object_id;
	SET curr_object_value = select object_value from objects_properties where oldrow.object_id = object_id and player_id = oldrow.player_id;
    SET is_merchant_sell = exists (select * from npcs where oldrow.object_from_npc = npc_id and npc_is_merchant = 'true');
    SET is_merchant_buy = exists (select * from npcs where newrow.object_from_npc = npc_id and npc_is_merchant = 'true');
    IF (oldrow.object_from_npc is not null and newrow.object_from_npc is null AND is_merchant_sell) THEN
        Update players set player_money = player_money - curr_object_value where player_id = oldrow.player_id;
        SET npc_n = select npc_name from npcs where oldrow.object_from_npc = npc_id;
        SET new_player_money = select player_money from players where newrow.player_id = player_id;
        SET trade_output = CHAR(13)+CHAR(10) || 'You buy the object ' || object_n || ' for ' || curr_object_value || ' money from the trader ' || npc_n || '.' || CHAR(13)+CHAR(10) || 'Your current amount of money: ' || new_player_money || 'money';
    ELSEIF (oldrow.object_from_npc is null and newrow.object_from_npc is not null AND is_merchant_buy) THEN
        Update players set player_money = player_money + curr_object_value where player_id = oldrow.player_id;
        SET npc_n = select npc_name from npcs where newrow.object_from_npc = npc_id;
        SET new_player_money = select player_money from players where newrow.player_id = player_id;
        SET trade_output = CHAR(13)+CHAR(10) || 'You sell the object ' || object_n || ' for ' || curr_object_value || ' money to the trader ' || npc_n || '.' || CHAR(13)+CHAR(10) || 'Your current amount of money: ' || new_player_money || 'money';
    END IF;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    IF trade_output <> '' THEN
        Insert into output_history values (newrow.player_id,max_output_id,trade_output,CURRENT_TIMESTAMP,true);
    END IF;
END;

CREATE TRIGGER CHECK_INSERT_TRADE_OFFER_BEFORE BEFORE INSERT ON trade_offers
    REFERENCING NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
	DECLARE min_object_value DECIMAL(7,2);
    DECLARE max_object_value DECIMAL(7,2);
	DECLARE is_merchant BOOLEAN;
	DECLARE is_in_same_room BOOLEAN;
	SET min_object_value = select o.object_initial_value * 0.8
	                       from objects_properties op, objects o
	                       where o.object_id = op.object_id and newrow.object_id = op.object_id and op.player_id = CURRENT_USER;
    SET max_object_value = select o.object_initial_value * 1.2
                       from objects_properties op, objects o
                       where o.object_id = op.object_id and newrow.object_id = op.object_id and op.player_id = CURRENT_USER;
    SET is_merchant = exists (select * from npcs where newrow.npc_id = npc_id and npc_is_merchant = 'true');
	SET is_in_same_room = exists (select * from npcs n ,players p where p.player_in_room = n.npc_in_room and p.player_id = CURRENT_USER and n.npc_id = newrow.npc_id);
	IF (min_object_value <= newrow.offer_value and max_object_value >= newrow.offer_value and is_merchant AND is_in_same_room) THEN
	    SET newrow.player_id = cast(CURRENT_USER as integer);
    ELSEIF (not is_in_same_room) THEN
        SIGNAL SQLSTATE '45002' SET MESSAGE_TEXT = 'Merchant isn´t in the same Room. Please make sure, the NPC is in the same room and you spell the NPC correctly.';
    ELSEIF (not is_merchant) THEN
        SIGNAL SQLSTATE '45003' SET MESSAGE_TEXT = 'The NPC isn´t merchant. You can´t trade with this NPC.';
    ELSEIF (min_object_value > newrow.offer_value) THEN
        SIGNAL SQLSTATE '45004' SET MESSAGE_TEXT = 'Merchant does not accept the offer because your offered price is too low. Try again!';
    ELSEIF (max_object_value < newrow.offer_value) THEN
        SIGNAL SQLSTATE '45004' SET MESSAGE_TEXT = 'Merchant does not accept the offer because your offered price is too high. Try again!';
    END IF;
END;

CREATE TRIGGER CHECK_INSERT_TRADE_OFFER_AFTER AFTER INSERT ON trade_offers
    REFERENCING NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE max_output_id integer;
    DECLARE trade_output varchar(200);
    DECLARE object_n varchar(50);
    DECLARE object_v decimal(7,2);
    DECLARE npc_n varchar(50);

    SET npc_n = select npc_name from npcs where newrow.npc_id = npc_id;
    SET object_n = select object_name from objects where object_id = newrow.object_id;

    Update objects_properties set object_value = newrow.offer_value where player_id = newrow.player_id and object_id = newrow.object_id;
    SET object_v = select object_value from objects_properties where object_id = newrow.object_id and player_id = newrow.player_id;
    SET trade_output = CHAR(13)+CHAR(10) || 'The merchant ' || npc_n || 'accepts the offer of || ' || object_v || ' money for the object ' || object_n || '.' || CHAR(13)+CHAR(10) || 'The price of an object can only be changed once.' || CHAR(13)+CHAR(10) || 'Now you can sell or buy the object.';

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    Insert into output_history values (newrow.player_id,max_output_id,trade_output,CURRENT_TIMESTAMP,true);
END;
-----------------------------

--- attack trigger -----
CREATE TRIGGER CHECK_INSERT_ENEMY_ATTACK_BEFORE BEFORE INSERT ON enemy_attack
    REFERENCING NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
	DECLARE enemy_is_defeated BOOLEAN;
	DECLARE is_in_same_room BOOLEAN;
	DECLARE object_in_inv BOOLEAN;
    SET enemy_is_defeated = exists (select * from enemy_fights where newrow.enemy_id = enemy_id and enemy_defeated = 'true' and player_id = CURRENT_USER);
	SET is_in_same_room = exists (select * from enemies e ,players p where p.player_in_room = e.enemy_in_room and p.player_id = CURRENT_USER and e.enemy_id = newrow.enemy_id);
	SET object_in_inv = exists (select 'true' from objects_properties where player_id = CURRENT_USER and ((object_in_inventory = 'true' and object_id = newrow.object_id) or newrow.object_id is null));
	IF (not enemy_is_defeated AND is_in_same_room AND object_in_inv) THEN
	    SET newrow.player_id = cast(CURRENT_USER as integer);
    ELSEIF (not is_in_same_room) THEN
        SIGNAL SQLSTATE '45006' SET MESSAGE_TEXT = 'Enemy isn´t in the same Room. Please make sure, the enemy is in the same room and you spell it correctly.';
    ELSEIF (enemy_is_defeated) THEN
        SIGNAL SQLSTATE '45007' SET MESSAGE_TEXT = 'Enemy is already dead. You can''t fight it again.';
    ELSEIF (not object_in_inv) THEN
        SIGNAL SQLSTATE '45008' SET MESSAGE_TEXT = 'Attack object isn`t in your inventory. Please make sure, you own the object and spell it correctly.';
    ELSE
        SIGNAL SQLSTATE '45009' SET MESSAGE_TEXT = 'Unkown Attack Error';
    END IF;
END;

CREATE TRIGGER CHECK_INSERT_ENEMY_ATTACK_AFTER AFTER INSERT ON enemy_attack
    REFERENCING NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE max_output_id INTEGER;
    DECLARE attack_output VARCHAR(2000) default '';
    DECLARE enemy_speed INTEGER;
    DECLARE enemy_lp INTEGER;
    DECLARE enemy_attack_points INTEGER;
    DECLARE enemy_defeated BOOLEAN;
    DECLARE player_lp INTEGER;
    DECLARE player_speed INTEGER;
    DECLARE player_attack_points INTEGER;
    DECLARE player_finance DECIMAL(7,2);
    DECLARE player_defeated BOOLEAN;
    DECLARE object_attack_points INTEGER;
    DECLARE reward_lpup INTEGER;
    DECLARE reward_atkup INTEGER;
    DECLARE reward_money DECIMAL(7,2);
    DECLARE reward_speed INTEGER;
    DECLARE stored_player_money INTEGER;
    SET enemy_speed = select enemy_attack_speed from enemies where enemy_id = newrow.enemy_id;
    SET enemy_attack_points = select enemy_attack from enemies where enemy_id = newrow.enemy_id;
    SET player_speed = select player_attack_speed from players where player_id = newrow.player_id;
    SET player_attack_points = select player_attack from players where player_id = newrow.player_id;
    SET player_finance = select player_money from player where player_id = newrow.player_id;
    SET object_attack_points = select object_attack from objects where object_id = newrow.object_id;
    SET reward_lpup = select enemy_when_defeated_lifepointsup from enemies where enemy_id = newrow.enemy_id;
    SET reward_atkup = select enemy_when_defeated_attackup from enemies where enemy_id = newrow.enemy_id;
    SET reward_money = select enemy_when_defeated_money from enemies where enemy_id = newrow.enemy_id;
    SET reward_speed = select enemy_when_defeated_speed from enemies where enemy_id = newrow.enemy_id;
    SET stored_player_money = select enemy_stored_player_money from enemy_fights where enemy_id = newrow.enemy_id and player_id = newrow.player_id;

    IF (object_attack_points is null) THEN
        SET object_attack_points = 0;
    END IF;

    IF (enemy_speed > player_speed) THEN
        update players set player_life_points = player_life_points - enemy_attack_points where player_id = newrow.player_id;
        SET player_lp = select player_life_points from players where player_id = newrow.player_id;
        SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'The enemy acts faster and attacks you first. You lose ' || enemy_attack_points || ' LP. Your current life points are ' || player_lp || ' LP.';
        SET player_defeated = exists (select player_life_points from players where player_id = newrow.player_id and player_life_points <= 0);
        IF (player_defeated) THEN
            IF (player_finance >= 3) THEN
                update players set player_money = player_money - 3 where player_id = newrow.player_id;
                update enemy_fights set enemy_stored_player_money = enemy_stored_player_money + 3 where enemy_id = newrow.enemy_id and player_id = newrow.player_id;
            ELSE
                update players set player_money = 0 where player_id = newrow.player_id;
                update enemy_fights set enemy_stored_player_money = enemy_stored_player_money + player_finance where enemy_id = newrow.enemy_id and player_id = newrow.player_id;
            END IF;
            update players set player_in_room = 'r1', player_life_points = (select game_initial_player_life_points from games where game_id = (select player_plays_game from players where player_id = newrow.player_id)) where player_id = newrow.player_id;
            SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'You were defeated by the enemy and you are now back at the starting point of your journey. Your life points has been reset to 100 LP and you lose some money to your enemy. You get the money back when you defeat the enemy.';
        ELSE
            update enemy_fights set enemy_current_life_points = enemy_current_life_points - player_attack_points - object_attack_points
                            where player_id = newrow.player_id and enemy_id = newrow.enemy_id;
            SET enemy_lp = select enemy_current_life_points from enemy_fights where player_id = newrow.player_id and enemy_id = newrow.enemy_id;
            SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'Now is your turn. The enemy lose ' || (player_attack_points + object_attack_points) || ' LP. Enemy current life points are ' || enemy_lp || ' LP.';
            SET enemy_defeated = exists (select enemy_current_life_points from enemy_fights where player_id = newrow.player_id and enemy_current_life_points <= 0 and enemy_id = newrow.enemy_id);
            IF (enemy_defeated) THEN
                update enemy_fights set enemy_defeated = 'true' where player_id = newrow.player_id and enemy_id = newrow.enemy_id;
                update objects_properties set object_is_visible = 'true' where player_id = newrow.player_id and object_from_enemy = newrow.enemy_id;
                update players set player_life_points = player_life_points + reward_lpup, player_attack = player_attack + reward_atkup,
                                   player_money = player_money + reward_money + stored_player_money, player_attack_speed = player_attack_speed + reward_speed
                               where player_id = newrow.player_id;
                update enemy_fights set ENEMY_STORED_PLAYER_MONEY = 0 where enemy_id = newrow.enemy_id and player_id = newrow.player_id;
                SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'Congrats. You defeated the enemy and received a reward. Reward: '|| reward_lpup || ' LP, ' || reward_atkup || ' attack points up, ' || reward_speed || ' attack speed up, ' || reward_money || ' money';
            END IF;
        END IF;
    ELSE
        update enemy_fights set enemy_current_life_points = enemy_current_life_points - player_attack_points - object_attack_points
                            where player_id = newrow.player_id and enemy_id = newrow.enemy_id;
        SET enemy_lp = select enemy_current_life_points from enemy_fights where player_id = newrow.player_id and enemy_id = newrow.enemy_id;
        SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'You acts faster and attacks the enemy first. The enemy lose ' || (player_attack_points + object_attack_points) || ' LP. Enemy current life points are ' || enemy_lp || ' LP.';
        SET enemy_defeated = exists (select enemy_current_life_points from enemy_fights where player_id = newrow.player_id and enemy_current_life_points <= 0 and enemy_id = newrow.enemy_id);
        IF (enemy_defeated) THEN
            update enemy_fights set enemy_defeated = 'true' where player_id = newrow.player_id and enemy_id = newrow.enemy_id;
            update objects_properties set object_is_visible = 'true' where player_id = newrow.player_id and object_from_enemy = newrow.enemy_id;
            update players set player_life_points = player_life_points + reward_lpup, player_attack = player_attack + reward_atkup,
                               player_money = player_money + reward_money + stored_player_money, player_attack_speed = player_attack_speed + reward_speed
                           where player_id = newrow.player_id;
            update enemy_fights set ENEMY_STORED_PLAYER_MONEY = 0 where enemy_id = newrow.enemy_id and player_id = newrow.player_id;
            SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'Congrats. You defeated the enemy and received a reward. Reward: '|| reward_lpup || ' LP, ' || reward_atkup || ' attack points up, ' || reward_speed || ' attack speed up, ' || reward_money || ' money';
        ELSE
            update players set player_life_points = player_life_points - enemy_attack_points where player_id = newrow.player_id;
            SET player_lp = select player_life_points from players where player_id = newrow.player_id;
            SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'Now the enemy attack you. You lose ' || enemy_attack_points || ' LP. Your current life points are ' || player_lp || ' LP.';
            SET player_defeated = exists (select player_life_points from players where player_id = newrow.player_id and player_life_points <= 0);
            IF (player_defeated) THEN
                IF (player_finance >= 3) THEN
                    update players set player_money = player_money - 3 where player_id = newrow.player_id;
                    update enemy_fights set enemy_stored_player_money = enemy_stored_player_money + 3 where enemy_id = newrow.enemy_id and player_id = newrow.player_id;
                ELSE
                    update players set player_money = 0 where player_id = newrow.player_id;
                    update enemy_fights set enemy_stored_player_money = enemy_stored_player_money + player_finance where enemy_id = newrow.enemy_id and player_id = newrow.player_id;
                END IF;
                update players set player_in_room = 'r1', player_life_points = (select game_initial_player_life_points from games where game_id = (select player_plays_game from players where player_id = newrow.player_id)) where player_id = newrow.player_id;
                SET attack_output = attack_output || CHAR(13)+CHAR(10) || 'You were defeated by the enemy and you are now back at the starting point of your journey. Your life points has been reset to 100 LP and you lose some money to your enemy. You get the money back when you defeat the enemy.';
            END IF;
        END IF;
    END IF;

    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    Insert into output_history values (newrow.player_id,max_output_id,attack_output,CURRENT_TIMESTAMP,true);
END;
-----------------------------

--- eat trigger -----
CREATE TRIGGER CHECK_UPDATE_OF_EAT_BEFORE BEFORE UPDATE OF object_eaten ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
	DECLARE object_not_eatable BOOLEAN;
	DECLARE in_inventory BOOLEAN;

	SET object_not_eatable = exists (select * from objects where 'false' = object_eatable and object_id = newrow.object_id);
	SET in_inventory = exists (select * from objects_properties where object_id = newrow.object_id and object_in_inventory = 'true' and player_id = newrow.player_id);

	IF (in_inventory) THEN
	    IF (object_not_eatable) THEN
	        SIGNAL SQLSTATE '45010' SET MESSAGE_TEXT = 'Object is not eatable. To check whether you can eat the object use the column ''object_eatable'' from table ''object''.';
	    ELSEIF (oldrow.object_eaten = 'false' and newrow.object_eaten = 'false') THEN
	        SIGNAL SQLSTATE '45011' SET MESSAGE_TEXT = 'Object was not eaten.';
	    ELSEIF (oldrow.object_eaten = 'true' and newrow.object_eaten = 'true') THEN
	        SIGNAL SQLSTATE '45012' SET MESSAGE_TEXT = 'Object has already been eaten before. To check whether you have already eaten the object use the column ''object_eaten'' from table ''object_properties''.';
	    ELSEIF (oldrow.object_eaten = 'true' and newrow.object_eaten = 'false') THEN
	        SIGNAL SQLSTATE '45013' SET MESSAGE_TEXT = 'Object cannot be made eatable again once it has been eaten.';
        END IF;
    ELSE
        SIGNAL SQLSTATE '45018' SET MESSAGE_TEXT = 'Object is not in your inventory. Please make sure, you own the object and spell it correctly.';
    END IF;
END;

CREATE TRIGGER CHECK_UPDATE_OF_EAT_AFTER AFTER UPDATE OF object_eaten ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE i integer default 0;
    DECLARE object_e_type varchar(20);
    DECLARE object_e_value integer;
    DECLARE output_eat varchar(200);
    DECLARE max_output_id integer;
    DECLARE object_n varchar(50);

    SET object_n = select object_name from objects o where object_id = newrow.object_id;

    SET output_eat = CHAR(13)+CHAR(10) || 'You eat the object ' || object_n || '.';
    FOR select * from OBJECT_EFFECTS where object_id = newrow.OBJECT_ID and OBJECT_EFFECT_FROM = 'eat' order by OBJECT_EFFECT_ID DO
        SET object_e_type = select OBJECT_EFFECT_TYPE from OBJECT_EFFECTS where object_id = newrow.OBJECT_ID and OBJECT_EFFECT_FROM = 'eat' order by OBJECT_EFFECT_ID limit 1 offset i;
        SET object_e_value = select OBJECT_EFFECT_VALUE from OBJECT_EFFECTS where object_id = newrow.OBJECT_ID and OBJECT_EFFECT_FROM = 'eat' order by OBJECT_EFFECT_ID limit 1 offset i;
        IF (i = 0) THEN
            SET output_eat = output_eat || CHAR(13)+CHAR(10) || 'You get: ';
        end if;
        IF (object_e_type = 'lp') THEN
            Update player set PLAYER_LIFE_POINTS = PLAYER_LIFE_POINTS + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_eat = output_eat || object_e_value || ' LP, ';
        ELSEIF (object_e_type = 'attack') THEN
            Update player set PLAYER_ATTACK = PLAYER_ATTACK + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_eat = output_eat || object_e_value || ' attack points, ';
        ELSEIF (object_e_type = 'speed') THEN
            Update player set PLAYER_ATTACK_SPEED = PLAYER_ATTACK_SPEED + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_eat = output_eat || object_e_value || ' speed, ';
        ELSEIF (object_e_type = 'money') THEN
            Update player set PLAYER_MONEY = PLAYER_MONEY + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_eat = output_eat || object_e_value || ' money, ';
        end if;
        SET i = i + 1;
    end for;

    SET output_eat = substring(output_eat,0,length(output_eat)-1);

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (newrow.player_id,max_output_id,output_eat,CURRENT_TIMESTAMP,true);
END;
-----------------------------

-- use trigger
CREATE TRIGGER CHECK_UPDATE_OF_USE_BEFORE BEFORE UPDATE OF object_magical_effect_used ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
	DECLARE object_not_usable BOOLEAN;
	DECLARE in_inventory BOOLEAN;

	SET object_not_usable = exists (select * from objects where 'false' = object_magical and object_id = newrow.object_id);
	SET in_inventory = exists (select * from objects_properties where object_id = newrow.object_id and object_in_inventory = 'true' and player_id = newrow.player_id);

	IF (in_inventory) THEN
        IF (object_not_usable) THEN
	        SIGNAL SQLSTATE '45014' SET MESSAGE_TEXT = 'Object is not usable.';
        ELSEIF (oldrow.object_magical_effect_used = 'false' and newrow.object_magical_effect_used = 'false') THEN
	        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'Object was not used.';
        ELSEIF (oldrow.object_magical_effect_used = 'true' and newrow.object_magical_effect_used = 'true') THEN
	        SIGNAL SQLSTATE '45016' SET MESSAGE_TEXT = 'Object has already been used before.';
        ELSEIF (oldrow.object_magical_effect_used = 'true' and newrow.object_magical_effect_used = 'false') THEN
	        SIGNAL SQLSTATE '45017' SET MESSAGE_TEXT = 'Object cannot be made usable again once it has been used.';
        END IF;
    ELSE
        SIGNAL SQLSTATE '45018' SET MESSAGE_TEXT = 'Object isn´t in your inventory. Please make sure, you own the object and spell it correctly.';
    END IF;
END;

CREATE TRIGGER CHECK_UPDATE_OF_USE_AFTER AFTER UPDATE OF object_magical_effect_used ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE i integer default 0;
    DECLARE object_e_type varchar(20);
    DECLARE object_e_value integer;
    DECLARE output_use varchar(200);
    DECLARE max_output_id integer;
    DECLARE object_n varchar(50);

    SET object_n = select object_name from objects o where object_id = newrow.object_id;
    SET output_use = CHAR(13)+CHAR(10) || 'You use the object ' || object_n || '.';
    FOR select * from OBJECT_EFFECTS where object_id = newrow.OBJECT_ID and OBJECT_EFFECT_FROM = 'use' order by OBJECT_EFFECT_ID DO
        SET object_e_type = select OBJECT_EFFECT_TYPE from OBJECT_EFFECTS where object_id = newrow.OBJECT_ID and OBJECT_EFFECT_FROM = 'use' order by OBJECT_EFFECT_ID limit 1 offset i;
        SET object_e_value = select OBJECT_EFFECT_VALUE from OBJECT_EFFECTS where object_id = newrow.OBJECT_ID and OBJECT_EFFECT_FROM = 'use' order by OBJECT_EFFECT_ID limit 1 offset i;
        IF (i = 0) THEN
            SET output_use = output_use || CHAR(13)+CHAR(10) || 'You get: ';
        end if;
        IF (object_e_type = 'lp') THEN
            Update player set PLAYER_LIFE_POINTS = PLAYER_LIFE_POINTS + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_use = output_use || object_e_value || ' LP, ';
        ELSEIF (object_e_type = 'attack') THEN
            Update player set PLAYER_ATTACK = PLAYER_ATTACK + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_use = output_use || object_e_value || ' attack points, ';
        ELSEIF (object_e_type = 'speed') THEN
            Update player set PLAYER_ATTACK_SPEED = PLAYER_ATTACK_SPEED + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_use = output_use || object_e_value || ' speed, ';
        ELSEIF (object_e_type = 'money') THEN
            Update player set PLAYER_MONEY = PLAYER_MONEY + object_e_value where player_id = newrow.PLAYER_ID;
            SET output_use = output_use || object_e_value || ' money, ';
        ELSEIF (object_e_type = 'light') THEN
            Update OBJECTS_PROPERTIES set OBJECT_SWITCHED_ON = 'true' where player_id = newrow.PLAYER_ID and OBJECT_ID = newrow.OBJECT_ID;
        end if;
        SET i = i + 1;
    end for;
    SET output_use = substring(output_use,0,length(output_use)-1);

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    IF object_e_type <> 'light' THEN
        Insert into output_history values (newrow.player_id,max_output_id,output_use,CURRENT_TIMESTAMP,true);
    END IF;
END;
-----------------------------

--- switched_on trigger -----
CREATE TRIGGER CHECK_UPDATE_OF_OBJECT_SWITCHED_ON_AFTER AFTER UPDATE OF object_switched_on ON objects_properties
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE output_use varchar(200);
    DECLARE max_output_id integer;
    DECLARE object_n varchar(50);

    SET object_n = select object_name from objects o where object_id = newrow.object_id;

    IF newrow.OBJECT_SWITCHED_ON = 'true' THEN
        SET output_use = CHAR(13)+CHAR(10) || 'You use the object ' || object_n || '.' || CHAR(13)+CHAR(10) || 'You switch the light on from the ' || object_n || '.' || ' Now you can take a closer look at dark rooms. ';
    ELSEIF newrow.OBJECT_SWITCHED_ON = 'false' THEN
        SET output_use = CHAR(13)+CHAR(10) || 'You use the object ' || object_n || '.' || CHAR(13)+CHAR(10) || 'You switch the light off from the ' || object_n || '.';
    end if;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (newrow.player_id,max_output_id,output_use,CURRENT_TIMESTAMP,true);
END;
-----------------------------

--- examine container trigger -----
CREATE TRIGGER CHECK_CONTAINER_EXAMINE_BEFORE BEFORE UPDATE OF container_is_examined ON container_examine
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE in_same_room BOOLEAN;
    DECLARE have_needed_object BOOLEAN;

    SET in_same_room = exists (select * from players where player_id = newrow.PLAYER_ID
                                and player_in_room = any(select CONTAINER_IN_ROOM from CONTAINERS where CONTAINER_ID = newrow.CONTAINER_ID));

    SET have_needed_object = exists (select * from CONTAINERS where CONTAINER_ID = newrow.CONTAINER_ID
                                                            and (CONTAINER_NEEDS_OBJECT is null or exists(select * from OBJECTS_PROPERTIES op
                                                                                                          where op.OBJECT_IN_INVENTORY = 'true'
                                                                                                            and op.PLAYER_ID = newrow.PLAYER_ID
                                                                                                            and CONTAINER_NEEDS_OBJECT = op.OBJECT_ID)));

    IF (not in_same_room) THEN
        SIGNAL SQLSTATE '45014' SET MESSAGE_TEXT = 'You are not in the same room of the room inventory.';
    ELSEIF (not have_needed_object) THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'You need a object in your inventory to open the room inventory.';
    END IF;
END;

CREATE TRIGGER CHECK_CONTAINER_EXAMINE_AFTER AFTER UPDATE OF container_is_examined ON container_examine
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE i integer default 0;
    DECLARE max_output_id integer;
    DECLARE examine_output varchar(5000) default '';
	DECLARE examined BOOLEAN;
    DECLARE object_n varchar(50);
    DECLARE object_a varchar(50);

    SET examined = exists (select container_is_examined from container_examine where container_id = newrow.container_id
                                                                                 and player_id = newrow.player_id
                                                                                 and CONTAINER_IS_EXAMINED = 'true'
                                                                                 and CONTAINER_IS_VISIBLE = 'true');

    IF examined THEN
        update objects_properties set object_is_visible = 'true'
        where object_in_container = any(select container_id from containers where container_id = newrow.CONTAINER_ID
                                and (container_needs_object is null or container_needs_object =
                                any(select object_id from objects_properties where object_in_inventory = 'true' and player_id = newrow.PLAYER_ID)))
                                and player_id = newrow.PLAYER_ID;
        FOR Select * from OBJECTS_PROPERTIES where OBJECT_IN_CONTAINER = newrow.CONTAINER_ID and PLAYER_ID = newrow.PLAYER_ID DO
            SET object_n = select o.object_name from object_properties op, objects o where op.object_id = o.object_id and op.OBJECT_IN_CONTAINER = newrow.CONTAINER_ID limit 1 offset i;
            SET object_a = select o.object_article from object_properties op, objects o where op.object_id = o.object_id and op.OBJECT_IN_CONTAINER = newrow.CONTAINER_ID limit 1 offset i;
            SET examine_output = examine_output || CHAR(13)+CHAR(10) || 'You found ' || object_a || ' ' || object_n || '.';
            SET i = i + 1;
        end for;

        IF examine_output = '' THEN
            SET examine_output = examine_output || CHAR(13)+CHAR(10) || 'You didn''t find anything in there.';
        end if;

        SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
        IF (max_output_id is null) THEN
            SET max_output_id = 1;
        END IF;
        Insert into output_history values (newrow.player_id,max_output_id,examine_output,CURRENT_TIMESTAMP,true);
    END IF;
END;
-----------------------------

-- quest trigger ------------------------------------------------------------------------------------------------------------------------------------------
CREATE TRIGGER CHECK_UPDATE_QUEST_RECEIVED_BEFORE BEFORE UPDATE OF quest_received ON QUESTS_RECEIVED
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE player_room varchar(10);
    DECLARE quest_npc_room varchar(10);

    SET player_room = select player_in_room from player where player_id = newrow.player_id;
    SET quest_npc_room = select npc_in_room from npc where npc_id = (select quest_from_npc from quests where quest_id = newrow.quest_id);
    IF player_room <> quest_npc_room THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'You are not in the same room as the (Quest)-NPC.';
    ELSEIF oldrow.quest_received = 'false' and newrow.quest_received = 'false' THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'You don''t accept you quest. You can start a quest by setting quest_received (column) to true in quest_received (table).';
    ELSEIF oldrow.quest_received = 'true' and newrow.quest_received = 'false' THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'You can''t cancel a started quest.';
    ELSEIF oldrow.quest_received = 'true' and newrow.quest_received = 'true' THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'The quest has already been received by you.';
    END IF;
END;

CREATE TRIGGER CHECK_UPDATE_QUEST_RECEIVED_AFTER AFTER UPDATE OF quest_received ON QUESTS_RECEIVED
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE max_output_id integer;
    DECLARE quest_output varchar(5000) default '';
    DECLARE quest_tit varchar(100);
    DECLARE quest_txt varchar(500);

    SET quest_tit = select quest_title from quests where quest_id = newrow.quest_id;
    SET quest_txt = select quest_text from quests where quest_id = newrow.quest_id;
    SET quest_output = CHAR(13)+CHAR(10) || 'The quest '''|| quest_tit || ''' start:' || CHAR(13)+CHAR(10) || '"' || quest_txt || '" Good luck solving!';

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (newrow.player_id,max_output_id,quest_output,CURRENT_TIMESTAMP,true);
END;

CREATE TRIGGER CHECK_UPDATE_QUEST_REWARD_BEFORE BEFORE UPDATE OF quest_reward_received ON QUESTS_RECEIVED
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE player_room varchar(10);
    DECLARE quest_npc_room varchar(10);

    SET player_room = select player_in_room from player where player_id = newrow.player_id;
    SET quest_npc_room = select npc_in_room from npc where npc_id = (select quest_from_npc from quests where quest_id = newrow.quest_id);

    IF player_room <> quest_npc_room THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'You are not in the same room as the (Quest)-NPC.';
    ELSEIF (newrow.quest_received = 'false') THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'The quest must started before you can receive the reward. You can start a quest by setting quest_received (column)to true in quest_received (table).';
    ELSEIF (newrow.quest_solved = 'false') THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'The quest must solved before you can received the reward. Check the task of the quest.';
    ELSEIF (oldrow.quest_reward_received = 'true' and newrow.quest_reward_received = 'true') THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'The reward for a quest can only be collected once.';
    ELSEIF (oldrow.quest_reward_received = 'true' and newrow.quest_reward_received = 'false') THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'The quest reward cannot be returned.';
    ELSEIF (newrow.quest_reward_received = 'false') THEN
        SIGNAL SQLSTATE '45015' SET MESSAGE_TEXT = 'You didn''t collect the reward.';
    end if;
END;

CREATE TRIGGER CHECK_UPDATE_QUEST_REWARD_AFTER AFTER UPDATE OF quest_reward_received ON QUESTS_RECEIVED
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE i integer default 0;
    DECLARE max_output_id integer;
    DECLARE quest_output varchar(5000) default '';
    DECLARE quest_tit varchar(100);
    DECLARE quest_reward_t varchar(100);
    DECLARE quest_e_value integer;
    DECLARE quest_e_object varchar(10);
    DECLARE object_n varchar(50);

    SET quest_tit = select quest_title from quests where quest_id = newrow.quest_id;
    FOR select * from quest_reward where quest_id = newrow.quest_id order by quest_reward_id DO
        SET quest_reward_t = select quest_reward_type from quest_reward where quest_id = newrow.quest_id order by quest_reward_id limit 1 offset i;
        SET quest_e_value = select quest_reward_value from quest_reward where quest_id = newrow.quest_id order by quest_reward_id limit 1 offset i;
        IF (i = 0) THEN
            SET quest_output = CHAR(13)+CHAR(10) || 'Congrats, rege! You completed the quest ''' || quest_tit || '''.' || CHAR(13)+CHAR(10) ||  'You get: ';
        end if;
        IF (quest_reward_t = 'lp') THEN
            Update player set PLAYER_LIFE_POINTS = PLAYER_LIFE_POINTS + quest_e_value where player_id = newrow.PLAYER_ID;
            SET quest_output = quest_output || quest_e_value || ' LP, ';
        ELSEIF (quest_reward_t = 'attack') THEN
            Update player set PLAYER_ATTACK = PLAYER_ATTACK + quest_e_value where player_id = newrow.PLAYER_ID;
            SET quest_output = quest_output || quest_e_value || ' attack points, ';
        ELSEIF (quest_reward_t = 'speed') THEN
            Update player set PLAYER_ATTACK_SPEED = PLAYER_ATTACK_SPEED + quest_e_value where player_id = newrow.PLAYER_ID;
            SET quest_output = quest_output || quest_e_value || ' speed, ';
        ELSEIF (quest_reward_t = 'money') THEN
            Update player set PLAYER_MONEY = PLAYER_MONEY + quest_e_value where player_id = newrow.PLAYER_ID;
            SET quest_output = quest_output || quest_e_value || ' money, ';
        ELSEIF (quest_reward_t = 'object') THEN
            SET quest_e_object = select quest_reward_object from quest_reward where quest_id = newrow.quest_id order by quest_reward_id limit 1 offset i;
            SET object_n = select object_name from objects where object_id = quest_e_object;
            Update objects_properties set object_in_inventory = 'true', object_from_npc = null, object_is_visible = 'true' where object_id = quest_e_object and PLAYER_ID = newrow.PLAYER_ID;
            SET quest_output = quest_output || 'object '|| object_n || ', ';
        end if;
        SET i = i + 1;
    end for;
    SET quest_output = substring(quest_output,0,length(quest_output)-1);

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (newrow.PLAYER_ID,max_output_id,quest_output,CURRENT_TIMESTAMP,true);
END;

-- quest: npc_need_object, object_in_room
CREATE TRIGGER CHECK_QUEST_SOLVED_BY_UPDATE_OBJECTS_PROPERTIES AFTER UPDATE ON OBJECTS_PROPERTIES
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE q_id varchar(10);
    DECLARE quest_tit varchar(100);
    DECLARE quest_npc_need_object boolean;
    DECLARE i integer default 0;
    DECLARE max_output_id integer;
    DECLARE quest_output varchar(5000) default '';

    FOR select * from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID DO
        SET q_id = select quest_id from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID limit 1 offset i;
        SET quest_tit = select quest_title from quests where quest_id = q_id;
        SET quest_npc_need_object = exists (select * from QUEST_REWARD where QUEST_ID = q_id
                                                                      and QUEST_SOLUTION_TYPE = 'npc_need_object'
                                                                      and exists (select * from OBJECTS_PROPERTIES
                                                                                  where QUEST_SOLUTION_SUBJECT like (OBJECT_FROM_NPC || '%')
                                                                                    and PLAYER_ID = newrow.PLAYER_ID
                                                                                    and QUEST_SOLUTION_SUBJECT like ('%' || OBJECT_ID || '%')));
        IF(quest_npc_need_object) THEN
            Update QUESTS_RECEIVED set QUEST_SOLVED = 'true' where PLAYER_ID = newrow.PLAYER_ID and QUEST_ID = q_id;
            Update OBJECTS_PROPERTIES set OBJECT_IS_VISIBLE = 'false'
            where PLAYER_ID = newrow.PLAYER_ID and exists (select * from QUEST_REWARD qr,QUESTS q, NPCS n
                                                           where qr.QUEST_SOLUTION_SUBJECT like ('%' || OBJECT_ID || '%')
                                                             and n.NPC_ID = q.QUEST_FROM_NPC and qr.QUEST_ID = q.QUEST_ID and OBJECT_FROM_NPC = n.NPC_ID);
            SET quest_output = quest_output || CHAR(13)+CHAR(10) || 'Congrats, you solved the quest ''' || quest_tit || '''. Claim your reward by update the ''quest_reward_received'' column in ''quest_update'' to the quest.';
        END IF;
        SET i = i + 1;
    END FOR;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    IF(quest_output <> '') THEN
    Insert into output_history values (newrow.PLAYER_ID,max_output_id,quest_output,CURRENT_TIMESTAMP,true);
    END IF;
END;

-- quest: player_in_room
CREATE TRIGGER CHECK_QUEST_SOLVED_BY_UPDATE_PLAYER AFTER UPDATE ON PLAYERS
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE q_id varchar(10);
    DECLARE quest_tit varchar(100);
    DECLARE quest_visited_room boolean;
    DECLARE i integer default 0;
    DECLARE max_output_id integer;
    DECLARE quest_output varchar(5000) default '';

    FOR select * from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID DO
        SET q_id = select quest_id from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID limit 1 offset i;
        SET quest_tit = select quest_title from quests where quest_id = q_id;
        SET quest_visited_room = exists (select * from QUEST_REWARD where QUEST_ID = q_id
                                                                   and QUEST_SOLUTION_TYPE = 'room_visited'
                                                                   and QUEST_SOLUTION_SUBJECT = any(select ROOM_ID from ROOMS_VISITED where newrow.PLAYER_ID = PLAYER_ID));
        IF(quest_visited_room) THEN
            Update QUESTS_RECEIVED set QUEST_SOLVED = 'true' where PLAYER_ID = newrow.PLAYER_ID and QUEST_ID = q_id;
            SET quest_output = quest_output || CHAR(13)+CHAR(10) || 'Congrats, you solved the quest ''' || quest_tit || '''. Claim your reward by update the ''quest_reward_received'' column in ''quest_update'' to the quest.';
        END IF;
        SET i = i + 1;
    END FOR;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    IF(quest_output <> '') THEN
        Insert into output_history values (newrow.PLAYER_ID,max_output_id,quest_output,CURRENT_TIMESTAMP,true);
    END IF;
END;

-- quest: enemy_defeated
CREATE TRIGGER CHECK_QUEST_SOLVED_BY_UPDATE_ENEMY_FIGHTS AFTER UPDATE ON ENEMY_FIGHTS
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE q_id varchar(10);
    DECLARE quest_tit varchar(100);
    DECLARE quest_enemy_defeated boolean;
    DECLARE i integer default 0;
    DECLARE max_output_id integer;
    DECLARE quest_output varchar(5000) default '';

    FOR select * from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID DO
        SET q_id = select quest_id from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID limit 1 offset i;
        SET quest_tit = select quest_title from quests where quest_id = q_id;
        SET quest_enemy_defeated = exists (select * from QUEST_REWARD where QUEST_ID = q_id
                                                                         and QUEST_SOLUTION_TYPE = 'enemy'
                                                                         and QUEST_SOLUTION_SUBJECT = any(select ENEMY_ID from ENEMY_FIGHTS where ENEMY_DEFEATED = 'true' and newrow.PLAYER_ID = PLAYER_ID));
        IF(quest_enemy_defeated) THEN
            Update QUESTS_RECEIVED set QUEST_SOLVED = 'true' where PLAYER_ID = newrow.PLAYER_ID and QUEST_ID = q_id;
            SET quest_output = quest_output || CHAR(13)+CHAR(10) || 'Congrats, you solved the quest ''' || quest_tit || '''. Claim your reward by update the ''quest_reward_received'' column in ''quest_update'' to the quest.';
        END IF;
        SET i = i + 1;
    END FOR;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    IF(quest_output <> '') THEN
        Insert into output_history values (newrow.PLAYER_ID,max_output_id,quest_output,CURRENT_TIMESTAMP,true);
    END IF;
END;

-- quest: trash_report
CREATE TRIGGER CHECK_QUEST_SOLVED_BY_DELETE_TRASH AFTER DELETE ON TRASH_REPORT
    REFERENCING OLD ROW AS oldrow FOR EACH ROW
BEGIN ATOMIC
DECLARE q_id varchar(10);
DECLARE quest_tit varchar(100);
DECLARE quest_trash_deleted boolean;
DECLARE i integer default 0;
DECLARE max_output_id integer;
DECLARE quest_output varchar(5000) default '';

    FOR select * from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = oldrow.PLAYER_ID DO
        SET q_id = select quest_id from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = oldrow.PLAYER_ID limit 1 offset i;
        SET quest_tit = select quest_title from quests where quest_id = q_id;
        SET quest_trash_deleted = not exists (select * from TRASH_REPORT where PLAYER_ID = oldrow.PLAYER_ID);

        IF(quest_trash_deleted) THEN
            Update QUESTS_RECEIVED set QUEST_SOLVED = 'true' where PLAYER_ID = oldrow.PLAYER_ID and QUEST_ID = q_id;
            SET quest_output = quest_output || CHAR(13)+CHAR(10) || 'Congrats, you solved the quest ''' || quest_tit || '''. Claim your reward by update the ''quest_reward_received'' column in ''quest_update'' to the quest.';
        END IF;
        SET i = i + 1;
    END FOR;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = oldrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    IF(quest_output <> '') THEN
        Insert into output_history values (oldrow.PLAYER_ID,max_output_id,quest_output,CURRENT_TIMESTAMP,true);
    END IF;
END;

-- quest solved by quest received
CREATE TRIGGER CHECK_QUEST_SOLVED_BY_UPDATE_QUEST_RECEIVED AFTER UPDATE ON QUESTS_RECEIVED
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
DECLARE q_id varchar(10);
DECLARE quest_tit varchar(100);
DECLARE quest_npc_need_object boolean;
DECLARE quest_visited_room boolean;
DECLARE quest_enemy_defeated boolean;
DECLARE quest_trash_deleted boolean;
DECLARE i integer default 0;
DECLARE max_output_id integer;
DECLARE quest_output varchar(5000) default '';

FOR select * from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID DO
        SET q_id = select quest_id from QUESTS_RECEIVED where QUEST_SOLVED = 'false' and QUEST_RECEIVED = 'true' and PLAYER_ID = newrow.PLAYER_ID limit 1 offset i;
        SET quest_tit = select quest_title from quests where quest_id = q_id;
        SET quest_npc_need_object = exists (select * from QUEST_REWARD where QUEST_ID = q_id
                                                                         and QUEST_SOLUTION_TYPE = 'npc_need_object'
                                                                         and exists (select * from OBJECTS_PROPERTIES
                                                                                     where QUEST_SOLUTION_SUBJECT like (OBJECT_FROM_NPC || '%')
                                                                                       and PLAYER_ID = newrow.PLAYER_ID
                                                                                       and QUEST_SOLUTION_SUBJECT like ('%' || OBJECT_ID || '%')));
        SET quest_visited_room = exists (select * from QUEST_REWARD where QUEST_ID = q_id
                                                                      and QUEST_SOLUTION_TYPE = 'room_visited'
                                                                      and QUEST_SOLUTION_SUBJECT = any(select ROOM_ID from ROOMS_VISITED where newrow.PLAYER_ID = PLAYER_ID));

        SET quest_enemy_defeated = exists (select * from QUEST_REWARD where QUEST_ID = q_id
                                                                        and QUEST_SOLUTION_TYPE = 'enemy'
                                                                        and QUEST_SOLUTION_SUBJECT = any(select ENEMY_ID from ENEMY_FIGHTS where ENEMY_DEFEATED = 'true' and newrow.PLAYER_ID = PLAYER_ID));

        SET quest_trash_deleted = not exists (select * from TRASH_REPORT where PLAYER_ID = newrow.PLAYER_ID);

        IF(quest_npc_need_object or quest_visited_room or quest_enemy_defeated or quest_trash_deleted) THEN
            Update QUESTS_RECEIVED set QUEST_SOLVED = 'true' where PLAYER_ID = newrow.PLAYER_ID and QUEST_ID = q_id;
            IF (quest_npc_need_object) THEN
                Update OBJECTS_PROPERTIES set OBJECT_IS_VISIBLE = 'false'
                where PLAYER_ID = newrow.PLAYER_ID and exists (select * from QUEST_REWARD qr,QUESTS q, NPCS n
                              where qr.QUEST_SOLUTION_SUBJECT like ('%' || OBJECT_ID || '%')
                                and n.NPC_ID = q.QUEST_FROM_NPC and qr.QUEST_ID = q.QUEST_ID and OBJECT_FROM_NPC = n.NPC_ID);
            end if;
            SET quest_output = quest_output || CHAR(13)+CHAR(10) || 'Congrats, you solved the quest ''' || quest_tit || '''. Claim your reward by update the ''quest_reward_received'' column in ''quest_update'' to the quest.';
        END IF;
        SET i = i + 1;
    END FOR;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;

    IF(quest_output <> '') THEN
        Insert into output_history values (newrow.PLAYER_ID,max_output_id,quest_output,CURRENT_TIMESTAMP,true);
    END IF;
END;

-- end quest trigger --------------------------------------------------------------------------------------------------------------------------------------------

-- check fontsize
CREATE TRIGGER CHECK_UPDATE_ON_PLAYER_FONT_SIZE_BEFORE BEFORE UPDATE OF PLAYER_FONT_SIZE ON players
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    IF newrow.PLAYER_FONT_SIZE <= 0 THEN
        SIGNAL SQLSTATE '45016' SET MESSAGE_TEXT = 'The font size must have a value higher than 0.';
    end if;
END

-- create fontsize output
CREATE TRIGGER CHECK_UPDATE_ON_PLAYER_FONT_SIZE_AFTER AFTER UPDATE OF PLAYER_FONT_SIZE ON players
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE max_output_id integer;
    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (newrow.player_id,max_output_id,CHAR(13)+CHAR(10) || 'Set font size to ' || newrow.PLAYER_FONT_SIZE || '.',CURRENT_TIMESTAMP,true);
END;

-- check theme style
CREATE TRIGGER CHECK_UPDATE_ON_PLAYER_THEME_STYLE_BEFORE BEFORE UPDATE OF PLAYER_THEME_STYLE ON players
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    IF newrow.PLAYER_THEME_STYLE <= 0 or newrow.PLAYER_THEME_STYLE > 2 THEN
        SIGNAL SQLSTATE '45017' SET MESSAGE_TEXT = 'This number does not match any existing theme style. The theme style number for white mode is 1. The theme style number for dark mode is 2.';
    end if;
END;

-- create theme style output
CREATE TRIGGER CHECK_UPDATE_ON_PLAYER_THEME_STYLE_AFTER AFTER UPDATE OF PLAYER_THEME_STYLE ON players
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE output_theme_style varchar(50) default '';
    DECLARE max_output_id integer;

    IF(newrow.PLAYER_THEME_STYLE = 1) THEN
        SET output_theme_style = CHAR(13)+CHAR(10) || 'Set theme style to white mode.';
    ELSEIF (newrow.PLAYER_THEME_STYLE = 2) THEN
        SET output_theme_style = CHAR(13)+CHAR(10) || 'Set theme style to dark mode.';
    end if;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (newrow.player_id,max_output_id,output_theme_style,CURRENT_TIMESTAMP,true);
END;

-- check table style
CREATE TRIGGER CHECK_UPDATE_ON_PLAYER_TABLE_STYLE_BEFORE BEFORE UPDATE OF PLAYER_TABLE_STYLE ON players
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    IF newrow.PLAYER_TABLE_STYLE <= 0 or newrow.PLAYER_TABLE_STYLE > 2 THEN
        SIGNAL SQLSTATE '45018' SET MESSAGE_TEXT = 'This number does not match any existing table style. The table style number for ''table style'' is 1. The table style number for ''row style'' is 2.';
    end if;
END;

-- create output table style
CREATE TRIGGER CHECK_UPDATE_ON_PLAYER_TABLE_STYLE_AFTER AFTER UPDATE OF PLAYER_TABLE_STYLE ON players
    REFERENCING OLD ROW AS oldrow NEW ROW AS newrow FOR EACH ROW
BEGIN ATOMIC
    DECLARE output_table_style varchar(50) default '';
    DECLARE max_output_id integer;

    IF(newrow.PLAYER_TABLE_STYLE = 1) THEN
        SET output_table_style = CHAR(13)+CHAR(10) || 'Set table style to ''table style''.';
    ELSEIF (newrow.PLAYER_TABLE_STYLE = 2) THEN
        SET output_table_style = CHAR(13)+CHAR(10) || 'Set table style to ''row style''.';
    end if;

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = newrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (newrow.player_id,max_output_id,output_table_style,CURRENT_TIMESTAMP,true);
END;

-- check trash in same room
CREATE TRIGGER CHECK_DELETE_FROM_TRASH_BEFORE AFTER DELETE ON TRASH_REPORT
    REFERENCING OLD ROW AS oldrow FOR EACH ROW
BEGIN ATOMIC
DECLARE player_room varchar(10);

    SET player_room = select PLAYER_IN_ROOM from players where PLAYER_ID = oldrow.PLAYER_ID;
    if(player_room <> oldrow.TRASH_IN_ROOM) THEN
        SIGNAL SQLSTATE '45018' SET MESSAGE_TEXT = 'The trash you want to remove is in another room.';
    end if;
END;

-- create trash remove output
CREATE TRIGGER CHECK_DELETE_FROM_TRASH_AFTER AFTER DELETE ON TRASH_REPORT
    REFERENCING OLD ROW AS oldrow FOR EACH ROW
BEGIN ATOMIC
DECLARE output_trash varchar(50) default '';
DECLARE max_output_id integer;

    SET output_trash = CHAR(13)+CHAR(10) || 'You remove ' || oldrow.TRASH || '.';

    SET max_output_id = select max(output_id) + 1 from output_history where player_id = oldrow.player_id;
    IF (max_output_id is null) THEN
        SET max_output_id = 1;
    END IF;
    Insert into output_history values (oldrow.player_id,max_output_id,output_trash,CURRENT_TIMESTAMP,true);
END;


------ DROP TRIGGER ------

DROP TRIGGER IF EXISTS INITIALIZE_GAME_FOR_NEW_PLAYER;
DROP TRIGGER IF EXISTS MAKE_ROOM_OBJECT_VISIBLE;
DROP TRIGGER IF EXISTS CHECK_IF_OBJECT_AND_PLAYER_IN_SAME_ROOM;
DROP TRIGGER IF EXISTS CHECK_UPDATE_OF_OBJECT_PLACE_AFTER;
DROP TRIGGER IF EXISTS CHECK_IF_PATH_TO_NEXT_ROOM_IS_AVAILABLE;
DROP TRIGGER IF EXISTS INSERT_VISITED_ROOM;

DROP TRIGGER IF EXISTS CHECK_TRADE_MONEY;
DROP TRIGGER IF EXISTS UPDATE_PLAYER_MONEY_AFTER_TRADE;
DROP TRIGGER IF EXISTS CHECK_INSERT_TRADE_OFFER_BEFORE;
DROP TRIGGER IF EXISTS CHECK_INSERT_TRADE_OFFER_AFTER;
DROP TRIGGER IF EXISTS CHECK_INSERT_ENEMY_ATTACK_BEFORE;
DROP TRIGGER IF EXISTS CHECK_INSERT_ENEMY_ATTACK_AFTER;

DROP TRIGGER IF EXISTS CHECK_UPDATE_OF_EAT_BEFORE;
DROP TRIGGER IF EXISTS CHECK_UPDATE_OF_EAT_AFTER;
DROP TRIGGER IF EXISTS CHECK_UPDATE_OF_USE_BEFORE;
DROP TRIGGER IF EXISTS CHECK_UPDATE_OF_USE_AFTER;
DROP TRIGGER IF EXISTS CHECK_UPDATE_OF_OBJECT_SWITCHED_ON_AFTER;

DROP TRIGGER IF EXISTS CHECK_CONTAINER_EXAMINE_BEFORE;
DROP TRIGGER IF EXISTS CHECK_CONTAINER_EXAMINE_AFTER;
--------
DROP TRIGGER IF EXISTS CHECK_UPDATE_QUEST_RECEIVED_BEFORE;
DROP TRIGGER IF EXISTS CHECK_UPDATE_QUEST_RECEIVED_AFTER;
DROP TRIGGER IF EXISTS CHECK_UPDATE_QUEST_REWARD_BEFORE;
DROP TRIGGER IF EXISTS CHECK_UPDATE_QUEST_REWARD_AFTER;
DROP TRIGGER IF EXISTS CHECK_QUEST_SOLVED_BY_UPDATE_OBJECTS_PROPERTIES;
DROP TRIGGER IF EXISTS CHECK_QUEST_SOLVED_BY_UPDATE_PLAYER;
DROP TRIGGER IF EXISTS CHECK_QUEST_SOLVED_BY_UPDATE_ENEMY_FIGHTS;
DROP TRIGGER IF EXISTS CHECK_QUEST_SOLVED_BY_DELETE_TRASH;
DROP TRIGGER IF EXISTS CHECK_QUEST_SOLVED_BY_UPDATE_QUEST_RECEIVED;
--------
DROP TRIGGER IF EXISTS CHECK_UPDATE_ON_PLAYER_FONT_SIZE_BEFORE;
DROP TRIGGER IF EXISTS CHECK_UPDATE_ON_PLAYER_FONT_SIZE_AFTER;
DROP TRIGGER IF EXISTS CHECK_UPDATE_ON_PLAYER_THEME_STYLE_BEFORE;
DROP TRIGGER IF EXISTS CHECK_UPDATE_ON_PLAYER_THEME_STYLE_AFTER;
DROP TRIGGER IF EXISTS CHECK_UPDATE_ON_PLAYER_TABLE_STYLE_BEFORE;
DROP TRIGGER IF EXISTS CHECK_UPDATE_ON_PLAYER_TABLE_STYLE_AFTER;
--------
DROP TRIGGER IF EXISTS CHECK_DELETE_FROM_TRASH_BEFORE;
DROP TRIGGER IF EXISTS CHECK_DELETE_FROM_TRASH_AFTER;